@@ -19,24 +19,26 @@
 #include "debugfs.h"
 
 struct block_info {
-	blk_t	blk;
-	ino_t	ino;
+	blk_t		blk;
+	ext2_ino_t	ino;
 };
 
 struct block_walk_struct {
 	struct block_info	*barray;
-	int			blocks_left;
-	int			num_blocks;
-	ino_t			inode;
+	e2_blkcnt_t		blocks_left;
+	e2_blkcnt_t		num_blocks;
+	ext2_ino_t		inode;
 };
 
 static int icheck_proc(ext2_filsys fs,
 		       blk_t	*block_nr,
-		       int blockcnt,
+		       e2_blkcnt_t blockcnt,
+		       blk_t ref_block,
+		       int ref_offset,
 		       void *private)
 {
 	struct block_walk_struct *bw = (struct block_walk_struct *) private;
-	int	i;
+	e2_blkcnt_t	i;
 
 	for (i=0; i < bw->num_blocks; i++) {
 		if (bw->barray[i].blk == *block_nr) {
@@ -56,10 +58,9 @@ void do_icheck(int argc, char **argv)
 	struct block_info	*binfo;
 	int			i;
 	ext2_inode_scan		scan = 0;
-	ino_t			ino;
+	ext2_ino_t		ino;
 	struct ext2_inode	inode;
 	errcode_t		retval;
-	char			*tmp;
 	char			*block_buf;
 	
 	if (argc < 2) {
@@ -84,11 +85,8 @@ void do_icheck(int argc, char **argv)
 	}
 
 	for (i=1; i < argc; i++) {
-		bw.barray[i-1].blk = strtol(argv[i], &tmp, 0);
-		if (*tmp) {
-			com_err(argv[0], 0, "Bad block - %s", argv[i]);
+		if (strtoblk(argv[0], argv[i], &bw.barray[i-1].blk))
 			return;
-		}
 	}
 
 	bw.num_blocks = bw.blocks_left = argc-1;
@@ -120,12 +118,12 @@ void do_icheck(int argc, char **argv)
 			goto next;
 
 		bw.inode = ino;
-		
-		retval = ext2fs_block_iterate(current_fs, ino, 0, block_buf,
-					      icheck_proc, &bw);
+
+		retval = ext2fs_block_iterate2(current_fs, ino, 0, block_buf,
+					       icheck_proc, &bw);
 		if (retval) {
 			com_err("icheck", retval,
-				"while calling ext2_block_iterate");
+				"while calling ext2fs_block_iterate");
 			goto next;
 		}
 
@@ -149,7 +147,7 @@ void do_icheck(int argc, char **argv)
 			printf("%u\t<block not found>\n", binfo->blk);
 			continue;
 		}
-		printf("%u\t%ld\n", binfo->blk, binfo->ino);
+		printf("%u\t%u\n", binfo->blk, binfo->ino);
 	}
 
 error_out:
