@@ -101,6 +101,8 @@ struct resource_track {
 #define E2F_OPT_TIME2		0x0020
 #define E2F_OPT_CHECKBLOCKS	0x0040
 #define E2F_OPT_DEBUG		0x0080
+#define E2F_OPT_FORCE		0x0100
+#define E2F_OPT_WRITECHECK	0x0200
 
 /*
  * E2fsck flags
@@ -115,6 +117,8 @@ struct resource_track {
 #define E2F_FLAG_PROG_BAR	0x0020 /* Progress bar on screen */
 #define E2F_FLAG_PROG_SUPPRESS	0x0040 /* Progress suspended */
 #define E2F_FLAG_JOURNAL_INODE	0x0080 /* Create a new ext3 journal inode */
+#define E2F_FLAG_SB_SPECIFIED	0x0100 /* The superblock was explicitly 
+					* specified by the user */
 
 /*
  * Defines for indicating the e2fsck pass number
@@ -126,6 +130,11 @@ struct resource_track {
 #define E2F_PASS_5	5
 #define E2F_PASS_1B	6
 
+/*
+ * Define the extended attribute refcount structure
+ */
+typedef struct ea_refcount *ext2_refcount_t;
+
 /*
  * This is the global e2fsck structure.
  */
@@ -140,6 +149,7 @@ struct e2fsck_struct {
 	int	options;
 	blk_t	use_superblock;	/* sb requested by user */
 	blk_t	superblock;	/* sb used to open fs */
+	int	blocksize;	/* blocksize */
 	blk_t	num_blocks;	/* Total number of blocks */
 
 #ifdef HAVE_SETJMP_H
@@ -159,6 +169,7 @@ struct e2fsck_struct {
 
 	ext2fs_block_bitmap block_found_map; /* Blocks which are in use */
 	ext2fs_block_bitmap block_dup_map; /* Blks referenced more than once */
+	ext2fs_block_bitmap block_ea_map; /* Blocks which are used by EA's */
 
 	/*
 	 * Inode count arrays
@@ -166,6 +177,9 @@ struct e2fsck_struct {
 	ext2_icount_t	inode_count;
 	ext2_icount_t inode_link_info;
 
+	ext2_refcount_t	refcount;
+	ext2_refcount_t refcount_extra;
+
 	/*
 	 * Array of flags indicating whether an inode bitmap, block
 	 * bitmap, or inode table is invalid
@@ -199,6 +213,12 @@ struct e2fsck_struct {
 	int process_inode_size;
 	int inode_buffer_blocks;
 
+	/*
+	 * ext3 journal support
+	 */
+	io_channel	journal_io;
+	const char	*journal_name;
+
 #ifdef RESOURCE_TRACK
 	/*
 	 * For timing purposes
@@ -231,6 +251,8 @@ struct e2fsck_struct {
 	int fs_tind_count;
 	int fs_fragmented;
 	int large_files;
+	int fs_ext_attr_inodes;
+	int fs_ext_attr_blocks;
 
 	/*
 	 * For the use of callers of the e2fsck functions; not used by
@@ -239,6 +261,9 @@ struct e2fsck_struct {
 	void *priv_data;
 };
 
+/* Used by the region allocation code */
+typedef __u32 region_addr_t;
+typedef struct region_struct *region_t;
 
 /*
  * Procedure declarations
@@ -271,6 +296,21 @@ extern int e2fsck_get_num_dirs(e2fsck_t ctx);
 extern int e2fsck_get_num_dirinfo(e2fsck_t ctx);
 extern struct dir_info *e2fsck_dir_info_iter(e2fsck_t ctx, int *control);
 
+/* ea_refcount.c */
+extern errcode_t ea_refcount_create(int size, ext2_refcount_t *ret);
+extern void ea_refcount_free(ext2_refcount_t refcount);
+extern errcode_t ea_refcount_fetch(ext2_refcount_t refcount, blk_t blk,
+				   int *ret);
+extern errcode_t ea_refcount_increment(ext2_refcount_t refcount,
+				       blk_t blk, int *ret);
+extern errcode_t ea_refcount_decrement(ext2_refcount_t refcount,
+				       blk_t blk, int *ret);
+extern errcode_t ea_refcount_store(ext2_refcount_t refcount,
+				   blk_t blk, int count);
+extern blk_t ext2fs_get_refcount_size(ext2_refcount_t refcount);
+extern void ea_refcount_intr_begin(ext2_refcount_t refcount);
+extern blk_t ea_refcount_intr_next(ext2_refcount_t refcount, int *ret);
+
 /* ehandler.c */
 extern const char *ehandler_operation(const char *op);
 extern void ehandler_init(io_channel channel);
@@ -278,10 +318,12 @@ extern void ehandler_init(io_channel channel);
 /* journal.c */
 extern int e2fsck_check_ext3_journal(e2fsck_t ctx);
 extern int e2fsck_run_ext3_journal(e2fsck_t ctx);
+extern void e2fsck_move_ext3_journal(e2fsck_t ctx);
 
 /* pass1.c */
 extern void e2fsck_use_inode_shortcuts(e2fsck_t ctx, int bool);
 extern int e2fsck_pass1_check_device_inode(struct ext2_inode *inode);
+extern int e2fsck_pass1_check_symlink(ext2_filsys fs, struct ext2_inode *inode);
 
 /* pass2.c */
 extern int e2fsck_process_bad_inode(e2fsck_t ctx, ext2_ino_t dir, ext2_ino_t ino);
@@ -289,6 +331,11 @@ extern int e2fsck_process_bad_inode(e2fsck_t ctx, ext2_ino_t dir, ext2_ino_t ino
 /* pass3.c */
 extern int e2fsck_reconnect_file(e2fsck_t ctx, ext2_ino_t inode);
 
+/* region.c */
+extern region_t region_create(region_addr_t min, region_addr_t max);
+extern void region_free(region_t region);
+extern int region_allocate(region_t region, region_addr_t start, int n);
+
 /* super.c */
 void check_super_block(e2fsck_t ctx);
 errcode_t e2fsck_get_device_size(e2fsck_t ctx);
@@ -318,7 +365,8 @@ extern void e2fsck_write_inode(e2fsck_t ctx, unsigned long ino,
 #ifdef MTRACE
 extern void mtrace_print(char *mesg);
 #endif
-extern blk_t get_backup_sb(ext2_filsys fs);
+extern blk_t get_backup_sb(e2fsck_t ctx, ext2_filsys fs,
+			   const char *name, io_manager manager);
 extern int ext2_file_type(unsigned int mode);
 
 /* unix.c */
