@@ -25,43 +25,62 @@
 #include "problem.h"
 #include "uuid/uuid.h"
 
-#ifdef JFS_DEBUG		/* Enabled by configure --enable-jfs-debug */
+#ifdef CONFIG_JBD_DEBUG		/* Enabled by configure --enable-jfs-debug */
 static int bh_count = 0;
 int journal_enable_debug = 2;
 #endif
 
+/*
+ * Define USE_INODE_IO to use the inode_io.c / fileio.c codepaths.
+ * This creates a larger static binary, and a smaller binary using
+ * shared libraries.  It's also probably slightly less CPU-efficient,
+ * which is why it's not on by default.  But, it's a good way of
+ * testing the functions in inode_io.c and fileio.c.
+ */
+#undef USE_INODE_IO
+
 /* Kernel compatibility functions for handling the journal.  These allow us
  * to use the recovery.c file virtually unchanged from the kernel, so we
  * don't have to do much to keep kernel and user recovery in sync.
  */
-int bmap(struct inode *inode, int block)
+int journal_bmap(journal_t *journal, blk_t block, unsigned long *phys)
 {
-	int retval;
-	blk_t phys;
+#ifdef USE_INODE_IO
+	*phys = block;
+	return 0;
+#else
+	struct inode 	*inode = journal->j_inode;
+	errcode_t	retval;
+	blk_t		pblk;
 
-	retval = ext2fs_bmap(inode->i_ctx->fs, inode->i_ino, &inode->i_ext2,
-			     NULL, 0, block, &phys);
+	if (!inode) {
+		*phys = block;
+		return 0;
+	}
 
-	if (retval)
-		com_err(inode->i_ctx->device_name, retval,
-			_("bmap journal inode %ld, block %d\n"),
-			inode->i_ino, block);
-
-	return phys;
+	retval= ext2fs_bmap(inode->i_ctx->fs, inode->i_ino, 
+			    &inode->i_ext2, NULL, 0, block, &pblk);
+	*phys = pblk;
+	return (retval);
+#endif
 }
 
-struct buffer_head *getblk(e2fsck_t ctx, blk_t blocknr, int blocksize)
+struct buffer_head *getblk(kdev_t kdev, blk_t blocknr, int blocksize)
 {
 	struct buffer_head *bh;
 
-	bh = e2fsck_allocate_memory(ctx, sizeof(*bh), "block buffer");
+	bh = e2fsck_allocate_memory(kdev->k_ctx, sizeof(*bh), "block buffer");
 	if (!bh)
 		return NULL;
 
 	jfs_debug(4, "getblk for block %lu (%d bytes)(total %d)\n",
 		  (unsigned long) blocknr, blocksize, ++bh_count);
 
-	bh->b_ctx = ctx;
+	bh->b_ctx = kdev->k_ctx;
+	if (kdev->k_dev == K_DEV_FS)
+		bh->b_io = kdev->k_ctx->fs->io;
+	else 
+		bh->b_io = kdev->k_ctx->journal_io;
 	bh->b_size = blocksize;
 	bh->b_blocknr = blocknr;
 
@@ -78,7 +97,7 @@ void ll_rw_block(int rw, int nr, struct buffer_head *bhp[])
 		if (rw == READ && !bh->b_uptodate) {
 			jfs_debug(3, "reading block %lu/%p\n", 
 				  (unsigned long) bh->b_blocknr, (void *) bh);
-			retval = io_channel_read_blk(bh->b_ctx->fs->io, 
+			retval = io_channel_read_blk(bh->b_io, 
 						     bh->b_blocknr,
 						     1, bh->b_data);
 			if (retval) {
@@ -92,7 +111,7 @@ void ll_rw_block(int rw, int nr, struct buffer_head *bhp[])
 		} else if (rw == WRITE && bh->b_dirty) {
 			jfs_debug(3, "writing block %lu/%p\n", 
 				  (unsigned long) bh->b_blocknr, (void *) bh);
-			retval = io_channel_write_blk(bh->b_ctx->fs->io, 
+			retval = io_channel_write_blk(bh->b_io, 
 						      bh->b_blocknr,
 						      1, bh->b_data);
 			if (retval) {
@@ -111,9 +130,9 @@ void ll_rw_block(int rw, int nr, struct buffer_head *bhp[])
 	}
 }
 
-void mark_buffer_dirty(struct buffer_head *bh, int dummy)
+void mark_buffer_dirty(struct buffer_head *bh)
 {
-	bh->b_dirty = dummy | 1; /* use dummy to avoid unused variable */
+	bh->b_dirty = 1;
 }
 
 static void mark_buffer_clean(struct buffer_head * bh)
@@ -157,118 +176,182 @@ static void e2fsck_clear_recover(e2fsck_t ctx, int error)
 	ext2fs_mark_super_dirty(ctx->fs);
 }
 
-static errcode_t e2fsck_journal_init_inode(e2fsck_t ctx,
-					   struct ext2_super_block *s,
-					   ext2_ino_t journal_inum,
-					   journal_t **journal)
+static errcode_t e2fsck_get_journal(e2fsck_t ctx, journal_t **ret_journal)
 {
-	struct inode *inode;
-	struct buffer_head *bh;
-	blk_t start;
-	int retval;
-
-	jfs_debug(1, "Using journal inode %u\n", journal_inum);
-	*journal = e2fsck_allocate_memory(ctx, sizeof(journal_t), "journal");
-	if (!*journal) {
+	struct ext2_super_block *sb = ctx->fs->super;
+	struct ext2_super_block jsuper;
+	struct problem_context	pctx;
+	struct buffer_head 	*bh;
+	struct inode		*j_inode = NULL;
+	struct kdev_s		*dev_fs = NULL, *dev_journal;
+	char			*journal_name = 0;
+	journal_t		*journal = NULL;
+	errcode_t		retval;
+	io_manager		io_ptr;
+	unsigned long		start = 0;
+	int			free_journal_name = 0;
+	int			ext_journal = 0;
+		
+	clear_problem_context(&pctx);
+	
+	journal = e2fsck_allocate_memory(ctx, sizeof(journal_t), "journal");
+	if (!journal) {
 		return EXT2_ET_NO_MEMORY;
 	}
 
-	inode = e2fsck_allocate_memory(ctx, sizeof(*inode), "journal inode");
-	if (!inode) {
+	dev_fs = e2fsck_allocate_memory(ctx, 2*sizeof(struct kdev_s), "kdev");
+	if (!dev_fs) {
 		retval = EXT2_ET_NO_MEMORY;
-		goto exit_journal;
+		goto errout;
 	}
+	dev_journal = dev_fs+1;
+	
+	dev_fs->k_ctx = dev_journal->k_ctx = ctx;
+	dev_fs->k_dev = K_DEV_FS;
+	dev_journal->k_dev = K_DEV_JOURNAL;
+	
+	journal->j_dev = dev_journal;
+	journal->j_fs_dev = dev_fs;
+	journal->j_inode = NULL;
+	journal->j_blocksize = ctx->fs->blocksize;
 
-	inode->i_ctx = ctx;
-	inode->i_ino = journal_inum;
-	retval = ext2fs_read_inode(ctx->fs, journal_inum, &inode->i_ext2);
-	if (retval)
-		goto exit_inode;
-
-	(*journal)->j_dev = ctx;
-	(*journal)->j_inode = inode;
-	(*journal)->j_blocksize = ctx->fs->blocksize;
-	(*journal)->j_maxlen = inode->i_ext2.i_size / (*journal)->j_blocksize;
-
-	if (!inode->i_ext2.i_links_count ||
-	    !LINUX_S_ISREG(inode->i_ext2.i_mode) ||
-	    (*journal)->j_maxlen < JFS_MIN_JOURNAL_BLOCKS ||
-	    (start = bmap(inode, 0)) == 0) {
-		retval = EXT2_ET_BAD_INODE_NUM;
-		goto exit_inode;
-	}
-
-	bh = getblk(ctx, start, (*journal)->j_blocksize);
-	if (!bh) {
-		retval = EXT2_ET_NO_MEMORY;
-		goto exit_inode;
-	}
-	(*journal)->j_sb_buffer = bh;
-	(*journal)->j_superblock = (journal_superblock_t *)bh->b_data;
-
-	return 0;
-
-exit_inode:
-	ext2fs_free_mem((void **)&inode);
-exit_journal:
-	ext2fs_free_mem((void **)journal);
-
-	return retval;
-}
-
-static errcode_t e2fsck_get_journal(e2fsck_t ctx, journal_t **journal)
-{
-	char uuid_str[40];
-	struct problem_context pctx;
-	struct ext2_super_block *sb = ctx->fs->super;
-
-	clear_problem_context(&pctx);
-
-	if (sb->s_feature_compat & EXT3_FEATURE_COMPAT_HAS_JOURNAL) {
-		/* FIXME: check if dev is valid block dev, has a journal */
-		if (sb->s_journal_dev) {
-			pctx.num = sb->s_journal_dev;
-			/* this problem aborts on -y, -p, unsupported on -n */
-			if (!fix_problem(ctx, PR_0_JOURNAL_UNSUPP_DEV, &pctx))
-				return EXT2_ET_UNSUPP_FEATURE;
-			sb->s_journal_dev = 0;
-			sb->s_state &= ~EXT2_VALID_FS;
-			ext2fs_mark_super_dirty(ctx->fs);
-		}
-		/* FIXME: check if UUID is valid block dev, has a journal */
-		if (!uuid_is_null(sb->s_journal_uuid)) {
-			uuid_unparse(sb->s_journal_uuid, uuid_str);
-			pctx.str = uuid_str;
-			/* this problem aborts on -y, -p, unsupported on -n */
-			if (!fix_problem(ctx, PR_0_JOURNAL_UNSUPP_UUID, &pctx))
-				return EXT2_ET_UNSUPP_FEATURE;
-			uuid_clear(sb->s_journal_uuid);
-			sb->s_state &= ~EXT2_VALID_FS;
-			ext2fs_mark_super_dirty(ctx->fs);
-		}
+	if (uuid_is_null(sb->s_journal_uuid)) {
 		if (!sb->s_journal_inum)
 			return EXT2_ET_BAD_INODE_NUM;
+		j_inode = e2fsck_allocate_memory(ctx, sizeof(*j_inode),
+						 "journal inode");
+		if (!j_inode) {
+			retval = EXT2_ET_NO_MEMORY;
+			goto errout;
+		}
+
+		j_inode->i_ctx = ctx;
+		j_inode->i_ino = sb->s_journal_inum;
+		
+		if ((retval = ext2fs_read_inode(ctx->fs,
+						sb->s_journal_inum,
+						&j_inode->i_ext2)))
+			goto errout;
+		if (!j_inode->i_ext2.i_links_count ||
+		    !LINUX_S_ISREG(j_inode->i_ext2.i_mode)) {
+			retval = EXT2_ET_NO_JOURNAL;
+			goto errout;
+		}
+		if (j_inode->i_ext2.i_size / journal->j_blocksize <
+		    JFS_MIN_JOURNAL_BLOCKS) {
+			retval = EXT2_ET_JOURNAL_TOO_SMALL;
+			goto errout;
+		}
+
+		journal->j_maxlen = j_inode->i_ext2.i_size / journal->j_blocksize;
+
+#ifdef USE_INODE_IO
+		retval = ext2fs_inode_io_intern(ctx->fs, sb->s_journal_inum,
+						&journal_name);
+		if (retval)
+			goto errout;
+
+		io_ptr = inode_io_manager;
+#else
+		journal->j_inode = j_inode;
+		ctx->journal_io = ctx->fs->io;
+		if ((retval = journal_bmap(journal, 0, &start)) != 0)
+			goto errout;
+#endif
+	} else {
+		ext_journal = 1;
+		journal_name = ctx->journal_name;
+		if (!journal_name) {
+			journal_name = ext2fs_find_block_device(sb->s_journal_dev);
+			free_journal_name = 1;
+		}
+			
+		if (!journal_name) {
+			fix_problem(ctx, PR_0_CANT_FIND_JOURNAL, &pctx);
+			return EXT2_ET_LOAD_EXT_JOURNAL;
+		}
+		
+		jfs_debug(1, "Using journal file %s\n", journal_name);
+		io_ptr = unix_io_manager;
 	}
 
-	if (sb->s_journal_dev) {
-		pctx.num = sb->s_journal_dev;
-		if (!fix_problem(ctx, PR_0_JOURNAL_BAD_DEV, &pctx))
-			return EXT2_ET_UNSUPP_FEATURE;
-		sb->s_journal_dev = 0;
-		sb->s_state &= ~EXT2_VALID_FS;
-		ext2fs_mark_super_dirty(ctx->fs);
-	}
-	if (!uuid_is_null(sb->s_journal_uuid)) {
-		uuid_unparse(sb->s_journal_uuid, uuid_str);
-		pctx.str = uuid_str;
-		if (!fix_problem(ctx, PR_0_JOURNAL_BAD_UUID, &pctx))
-			return EXT2_ET_UNSUPP_FEATURE;
-		uuid_clear(sb->s_journal_uuid);
-		sb->s_state &= ~EXT2_VALID_FS;
-		ext2fs_mark_super_dirty(ctx->fs);
+#if 0
+	test_io_backing_manager = io_ptr;
+	io_ptr = test_io_manager;
+#endif
+#ifndef USE_INODE_IO
+	if (ext_journal)
+#endif
+		retval = io_ptr->open(journal_name, IO_FLAG_RW,
+				      &ctx->journal_io);
+	if (free_journal_name)
+		free((void *) journal_name);
+	if (retval)
+		goto errout;
+
+	io_channel_set_blksize(ctx->journal_io, ctx->fs->blocksize);
+
+	if (ext_journal) {
+		if (ctx->fs->blocksize == 1024)
+			start = 1;
+		bh = getblk(dev_journal, start, ctx->fs->blocksize);
+		if (!bh) {
+			retval = EXT2_ET_NO_MEMORY;
+			goto errout;
+		}
+		ll_rw_block(READ, 1, &bh);
+		if ((retval = bh->b_err) != 0)
+			goto errout;
+		memcpy(&jsuper, start ? bh->b_data :  bh->b_data + 1024,
+		       sizeof(jsuper));
+		brelse(bh);
+#ifdef EXT2FS_ENABLE_SWAPFS
+		if (jsuper.s_magic == ext2fs_swab16(EXT2_SUPER_MAGIC)) 
+			ext2fs_swap_super(&jsuper);
+#endif
+		if (jsuper.s_magic != EXT2_SUPER_MAGIC ||
+		    !(jsuper.s_feature_incompat & EXT3_FEATURE_INCOMPAT_JOURNAL_DEV)) {
+			fix_problem(ctx, PR_0_EXT_JOURNAL_BAD_SUPER, &pctx);
+			retval = EXT2_ET_LOAD_EXT_JOURNAL;
+			goto errout;
+		}
+		/* Make sure the journal UUID is correct */
+		if (memcmp(jsuper.s_uuid, ctx->fs->super->s_journal_uuid,
+			   sizeof(jsuper.s_uuid))) {
+			fix_problem(ctx, PR_0_JOURNAL_BAD_UUID, &pctx);
+			retval = EXT2_ET_LOAD_EXT_JOURNAL;
+			goto errout;
+		}
+		
+		journal->j_maxlen = jsuper.s_blocks_count;
+		start++;
 	}
 
-	return e2fsck_journal_init_inode(ctx, sb, sb->s_journal_inum, journal);
+	if (!(bh = getblk(dev_journal, start, journal->j_blocksize))) {
+		retval = EXT2_ET_NO_MEMORY;
+		goto errout;
+	}
+	
+	journal->j_sb_buffer = bh;
+	journal->j_superblock = (journal_superblock_t *)bh->b_data;
+	
+#ifdef USE_INODE_IO
+	if (j_inode)
+		ext2fs_free_mem((void **)&j_inode);
+#endif
+
+	*ret_journal = journal;
+	return 0;
+
+errout:
+	if (dev_fs)
+		ext2fs_free_mem((void **)&dev_fs);
+	if (j_inode)
+		ext2fs_free_mem((void **)&j_inode);
+	if (journal)
+		ext2fs_free_mem((void **)&journal);
+	return retval;
+	
 }
 
 static errcode_t e2fsck_journal_fix_bad_inode(e2fsck_t ctx,
@@ -304,9 +387,26 @@ static errcode_t e2fsck_journal_fix_bad_inode(e2fsck_t ctx,
 	return 0;
 }
 
+#define V1_SB_SIZE	0x0024
+static void clear_v2_journal_fields(journal_t *journal)
+{
+	e2fsck_t ctx = journal->j_dev->k_ctx;
+	struct problem_context pctx;
+
+	clear_problem_context(&pctx);
+
+	if (!fix_problem(ctx, PR_0_CLEAR_V2_JOURNAL, &pctx))
+		return;
+
+	memset(((char *) journal->j_superblock) + V1_SB_SIZE, 0,
+	       ctx->fs->blocksize-V1_SB_SIZE);
+	mark_buffer_dirty(journal->j_sb_buffer);
+}
+
+
 static errcode_t e2fsck_journal_load(journal_t *journal)
 {
-	e2fsck_t ctx = journal->j_dev;
+	e2fsck_t ctx = journal->j_dev->k_ctx;
 	journal_superblock_t *jsb;
 	struct buffer_head *jbh = journal->j_sb_buffer;
 	struct problem_context pctx;
@@ -328,10 +428,22 @@ static errcode_t e2fsck_journal_load(journal_t *journal)
 	switch (ntohl(jsb->s_header.h_blocktype)) {
 	case JFS_SUPERBLOCK_V1:
 		journal->j_format_version = 1;
+		if (jsb->s_feature_compat ||
+		    jsb->s_feature_incompat ||
+		    jsb->s_feature_ro_compat ||
+		    jsb->s_nr_users)
+			clear_v2_journal_fields(journal);
 		break;
 		
 	case JFS_SUPERBLOCK_V2:
 		journal->j_format_version = 2;
+		if (ntohl(jsb->s_nr_users) > 1 &&
+		    (ctx->fs->io == ctx->journal_io))
+			clear_v2_journal_fields(journal);
+		if (ntohl(jsb->s_nr_users) > 1) {
+			fix_problem(ctx, PR_0_JOURNAL_UNSUPP_MULTIFS, &pctx);
+			return EXT2_ET_JOURNAL_UNSUPP_VERSION;
+		}
 		break;
 
 	/*
@@ -424,7 +536,7 @@ static void e2fsck_journal_reset_super(e2fsck_t ctx, journal_superblock_t *jsb,
 		new_seq ^= u.val[i];
 	jsb->s_sequence = htonl(new_seq);
 
-	mark_buffer_dirty(journal->j_sb_buffer, 1);
+	mark_buffer_dirty(journal->j_sb_buffer);
 	ll_rw_block(WRITE, 1, &journal->j_sb_buffer);
 }
 
@@ -436,8 +548,6 @@ static errcode_t e2fsck_journal_fix_corrupt_super(e2fsck_t ctx,
 	int recover = ctx->fs->super->s_feature_incompat &
 		EXT3_FEATURE_INCOMPAT_RECOVER;
 
-	pctx->num = journal->j_inode->i_ino;
-
 	if (sb->s_feature_compat & EXT3_FEATURE_COMPAT_HAS_JOURNAL) {
 		if (fix_problem(ctx, PR_0_JOURNAL_BAD_SUPER, pctx)) {
 			e2fsck_journal_reset_super(ctx, journal->j_superblock,
@@ -465,12 +575,22 @@ static void e2fsck_journal_release(e2fsck_t ctx, journal_t *journal,
 		jsb->s_sequence = htonl(journal->j_transaction_sequence);
 		if (reset)
 			jsb->s_start = 0; /* this marks the journal as empty */
-		mark_buffer_dirty(journal->j_sb_buffer, 1);
+		mark_buffer_dirty(journal->j_sb_buffer);
 	}
 	brelse(journal->j_sb_buffer);
 
+	if (ctx->journal_io) {
+		if (ctx->fs && ctx->fs->io != ctx->journal_io)
+			io_channel_close(ctx->journal_io);
+		ctx->journal_io = 0;
+	}
+	
+#ifndef USE_INODE_IO
 	if (journal->j_inode)
 		ext2fs_free_mem((void **)&journal->j_inode);
+#endif
+	if (journal->j_fs_dev)
+		ext2fs_free_mem((void **)&journal->j_fs_dev);
 	ext2fs_free_mem((void **)&journal);
 }
 
@@ -485,6 +605,7 @@ int e2fsck_check_ext3_journal(e2fsck_t ctx)
 	int recover = ctx->fs->super->s_feature_incompat &
 		EXT3_FEATURE_INCOMPAT_RECOVER;
 	struct problem_context pctx;
+	problem_t problem;
 	int reset = 0, force_fsck = 0;
 	int retval;
 
@@ -499,7 +620,9 @@ int e2fsck_check_ext3_journal(e2fsck_t ctx)
 
 	retval = e2fsck_get_journal(ctx, &journal);
 	if (retval) {
-		if (retval == EXT2_ET_BAD_INODE_NUM)
+		if ((retval == EXT2_ET_BAD_INODE_NUM) ||
+		    (retval == EXT2_ET_JOURNAL_TOO_SMALL) ||
+		    (retval == EXT2_ET_NO_JOURNAL))
 			return e2fsck_journal_fix_bad_inode(ctx, &pctx);
 		return retval;
 	}
@@ -555,7 +678,19 @@ no_has_journal:
 	if (sb->s_feature_compat & EXT3_FEATURE_COMPAT_HAS_JOURNAL &&
 	    !(sb->s_feature_incompat & EXT3_FEATURE_INCOMPAT_RECOVER) &&
 	    journal->j_superblock->s_start != 0) {
-		if (fix_problem(ctx, PR_0_JOURNAL_RESET_JOURNAL, &pctx)) {
+		/* Print status information */
+		fix_problem(ctx, PR_0_JOURNAL_RECOVERY_CLEAR, &pctx);
+		if (ctx->superblock)
+			problem = PR_0_JOURNAL_RUN_DEFAULT;
+		else
+			problem = PR_0_JOURNAL_RUN;
+		if (fix_problem(ctx, problem, &pctx)) {
+			ctx->options |= E2F_OPT_FORCE;
+			sb->s_feature_incompat |=
+				EXT3_FEATURE_INCOMPAT_RECOVER;
+			ext2fs_mark_super_dirty(ctx->fs);
+		} else if (fix_problem(ctx,
+				       PR_0_JOURNAL_RESET_JOURNAL, &pctx)) {
 			reset = 1;
 			sb->s_state &= ~EXT2_VALID_FS;
 			ext2fs_mark_super_dirty(ctx->fs);
@@ -582,6 +717,7 @@ static errcode_t recover_ext3_journal(e2fsck_t ctx)
 	journal_t *journal;
 	int retval;
 
+	journal_init_revoke_caches();
 	retval = e2fsck_get_journal(ctx, &journal);
 	if (retval)
 		return retval;
@@ -595,114 +731,23 @@ static errcode_t recover_ext3_journal(e2fsck_t ctx)
 		goto errout;
 	
 	retval = -journal_recover(journal);
+	if (retval)
+		goto errout;
+	
+	if (journal->j_superblock->s_errno) {
+		ctx->fs->super->s_state |= EXT2_ERROR_FS;
+		ext2fs_mark_super_dirty(ctx->fs);
+		journal->j_superblock->s_errno = 0;
+		mark_buffer_dirty(journal->j_sb_buffer);
+	}
+		
 errout:
+	journal_destroy_revoke(journal);
+	journal_destroy_revoke_caches();
 	e2fsck_journal_release(ctx, journal, 1, 0);
 	return retval;
 }
 
-
-#if 0
-#define TEMPLATE "/tmp/ext3.XXXXXX"
-
-/*
- * This function attempts to mount and unmount an ext3 filesystem,
- * which is a cheap way to force the kernel to run the journal and
- * handle the recovery for us.
- */
-static errcode_t recover_ext3_journal_via_mount(e2fsck_t ctx)
-{
-	ext2_filsys fs = ctx->fs;
-	char	*dirlist[] = {"/mnt","/lost+found","/tmp","/root","/boot",0};
-	errcode_t	 retval, retval2;
-	int	 count = 0;
-	char	 template[] = TEMPLATE;
-	struct stat buf;
-	char	*tmpdir;
-
-	if (ctx->options & E2F_OPT_READONLY) {
-		printf("%s: won't do journal recovery while read-only\n",
-		       ctx->device_name);
-		return EXT2_ET_FILE_RO;
-	}
-
-	printf(_("%s: trying for ext3 kernel journal recovery\n"),
-	       ctx->device_name);
-	/*
-	 * First try to make a temporary directory.  This may fail if
-	 * the root partition is still mounted read-only.
-	 */
-newtemp:
-	tmpdir = mktemp(template);
-	if (tmpdir) {
-		jfs_debug(2, "trying %s as ext3 temp mount point\n", tmpdir);
-		if (mkdir(template, 0700)) {
-			if (errno == EROFS) {
-				tmpdir = NULL;
-				template[0] = '\0';
-			} else if (errno == EEXIST && count++ < 10) {
-				strcpy(template, TEMPLATE);
-				goto newtemp;
-			}
-			return errno;
-		}
-	}
-
-	/*
-	 * OK, creating a temporary directory didn't work.
-	 * Let's try a list of possible temporary mountpoints.
-	 */
-	if (!tmpdir) {
-		dev_t	rootdev;
-		char	**cpp, *dir;
-
-		if (stat("/", &buf))
-			return errno;
-
-		rootdev = buf.st_dev;
-
-		/*
-		 * Check that dir is on the same device as root (no other
-		 * filesystem is mounted there), and it's a directory.
-		 */
-		for (cpp = dirlist; (dir = *cpp); cpp++)
-			if (stat(dir, &buf) == 0 && buf.st_dev == rootdev &&
-			    S_ISDIR(buf.st_mode)) {
-				tmpdir = dir;
-				break;
-			}
-	}
-
-	if (tmpdir) {
-		io_manager	io_ptr = fs->io->manager;
-		int		blocksize = fs->blocksize;
-
-		jfs_debug(2, "using %s for ext3 mount\n", tmpdir);
-		/* FIXME - need to handle loop devices here */
-		if (mount(ctx->device_name, tmpdir, "ext3", MNT_FL, NULL)) {
-			retval = errno;
-			com_err(ctx->program_name, errno,
-				"when mounting %s", ctx->device_name);
-			if (template[0])
-				rmdir(tmpdir);
-			return retval;
-		}
-		/*
-		 * Now that it mounted cleanly, the filesystem will have been
-		 * recovered, so we can now unmount it.
-		 */
-		if (umount(tmpdir))
-			return errno;
-
-		/*
-		 * Remove the temporary directory, if it was created.
-		 */
-		if (template[0])
-			rmdir(tmpdir);
-		return 0;
-	}
-}
-#endif
-
 int e2fsck_run_ext3_journal(e2fsck_t ctx)
 {
 	io_manager io_ptr = ctx->fs->io->manager;
@@ -716,7 +761,8 @@ int e2fsck_run_ext3_journal(e2fsck_t ctx)
 		return EXT2_ET_FILE_RO;
 	}
 
-	ext2fs_flush(ctx->fs);	/* Force out any modifications */
+	if (ctx->fs->flags & EXT2_FLAG_DIRTY)
+		ext2fs_flush(ctx->fs);	/* Force out any modifications */
 
 	recover_retval = recover_ext3_journal(ctx);
 	
@@ -741,3 +787,103 @@ int e2fsck_run_ext3_journal(e2fsck_t ctx)
 	e2fsck_clear_recover(ctx, recover_retval);
 	return recover_retval;
 }
+
+/*
+ * This function will move the journal inode from a visible file in
+ * the filesystem directory hierarchy to the reserved inode if necessary.
+ */
+static const char * const journal_names[] = {
+	".journal", "journal", ".journal.dat", "journal.dat", 0 };
+
+void e2fsck_move_ext3_journal(e2fsck_t ctx)
+{
+	struct ext2_super_block *sb = ctx->fs->super;
+	struct problem_context	pctx;
+	struct ext2_inode 	inode;
+	ext2_filsys		fs = ctx->fs;
+	ext2_ino_t		ino;
+	errcode_t		retval;
+	const char * const *	cpp;
+	int			group, mount_flags;
+	
+	/*
+	 * If the filesystem is opened read-only, or there is no
+	 * journal, or the journal is already in the hidden inode,
+	 * then do nothing.
+	 */
+	if ((ctx->options & E2F_OPT_READONLY) ||
+	    (sb->s_journal_inum == 0) ||
+	    (sb->s_journal_inum == EXT2_JOURNAL_INO) ||
+	    !(sb->s_feature_compat & EXT3_FEATURE_COMPAT_HAS_JOURNAL))
+		return;
+
+	/*
+	 * If the filesystem is mounted, or we can't tell whether
+	 * or not it's mounted, do nothing.
+	 */
+	retval = ext2fs_check_if_mounted(ctx->filesystem_name, &mount_flags);
+	if (retval || (mount_flags & EXT2_MF_MOUNTED))
+		return;
+
+	/*
+	 * If we can't find the name of the journal inode, then do
+	 * nothing.
+	 */
+	for (cpp = journal_names; *cpp; cpp++) {
+		retval = ext2fs_lookup(fs, EXT2_ROOT_INO, *cpp,
+				       strlen(*cpp), 0, &ino);
+		if ((retval == 0) && (ino == sb->s_journal_inum))
+			break;
+	}
+	if (*cpp == 0)
+		return;
+
+	/*
+	 * The inode had better have only one link and not be readable.
+	 */
+	if (ext2fs_read_inode(fs, ino, &inode) != 0)
+		return;
+	if (inode.i_links_count != 1)
+		return;
+
+	/* We need the inode bitmap to be loaded */
+	retval = ext2fs_read_bitmaps(fs);
+	if (retval)
+		return;
+
+	clear_problem_context(&pctx);
+	pctx.str = *cpp;
+	if (!fix_problem(ctx, PR_0_MOVE_JOURNAL, &pctx))
+		return;
+		
+	/*
+	 * OK, we've done all the checks, let's actually move the
+	 * journal inode.  Errors at this point mean we need to force
+	 * an ext2 filesystem check.
+	 */
+	if ((retval = ext2fs_unlink(fs, EXT2_ROOT_INO, *cpp, ino, 0)) != 0)
+		goto err_out;
+	if ((retval = ext2fs_write_inode(fs, EXT2_JOURNAL_INO, &inode)) != 0)
+		goto err_out;
+	sb->s_journal_inum = EXT2_JOURNAL_INO;
+	ext2fs_mark_super_dirty(fs);
+	inode.i_links_count = 0;
+	inode.i_dtime = time(0);
+	if ((retval = ext2fs_write_inode(fs, ino, &inode)) != 0)
+		goto err_out;
+
+	group = ext2fs_group_of_ino(fs, ino);
+	ext2fs_unmark_inode_bitmap(fs->inode_map, ino);
+	ext2fs_mark_ib_dirty(fs);
+	fs->group_desc[group].bg_free_inodes_count++;
+	fs->super->s_free_inodes_count++;
+	return;
+
+err_out:
+	pctx.errcode = retval;
+	fix_problem(ctx, PR_0_ERR_MOVE_JOURNAL, &pctx);
+	fs->super->s_state &= ~EXT2_VALID_FS;
+	ext2fs_mark_super_dirty(fs);
+	return;
+}
+
