@@ -16,12 +16,12 @@
  * 	%b	<blk>			block number
  * 	%B	<blkcount>		integer
  * 	%c	<blk2>			block number
- * 	%di	<dirent>->ino		inode number
- * 	%dn	<dirent>->name		string
- * 	%dr	<dirent>->rec_len
- * 	%dl	<dirent>->name_len
- * 	%dt	<dirent>->filetype
- * 	%D	<dir> 			inode number
+ * 	%Di	<dirent>->ino		inode number
+ * 	%Dn	<dirent>->name		string
+ * 	%Dr	<dirent>->rec_len
+ * 	%Dl	<dirent>->name_len
+ * 	%Dt	<dirent>->filetype
+ * 	%d	<dir> 			inode number
  * 	%g	<group>			integer
  * 	%i	<ino>			inode number
  * 	%Is	<inode> -> i_size
@@ -50,6 +50,7 @@
  *
  * The following '@' expansions are supported:
  *
+ * 	@a	extended attribute
  * 	@A	error allocating
  * 	@b	block
  * 	@B	bitmap
@@ -99,6 +100,7 @@
  * letter <i> in the table below.
  */
 static const char *abbrevs[] = {
+	N_("aextended attribute"),
 	N_("Aerror allocating"),
 	N_("bblock"),
 	N_("Bbitmap"),
@@ -394,7 +396,7 @@ static _INLINE_ void expand_percent_expression(ext2_filsys fs, char ch,
 		print_pathname(fs, ctx->dir, ctx->ino);
 		break;
 	case 'S':
-		printf("%d", get_backup_sb(fs));
+		printf("%d", get_backup_sb(NULL, fs, NULL, NULL));
 		break;
 	case 's':
 		printf("%s", ctx->str ? ctx->str : "NULL");
