@@ -43,6 +43,8 @@
 #endif
 
 #include "e2fsck.h"
+#include <ext2fs/ext2_ext_attr.h>
+
 #include "problem.h"
 
 #ifdef NO_INLINE_FUNCS
@@ -68,6 +70,8 @@ static void process_inodes(e2fsck_t ctx, char *block_buf);
 static EXT2_QSORT_TYPE process_inode_cmp(const void *a, const void *b);
 static errcode_t scan_callback(ext2_filsys fs, ext2_inode_scan scan,
 				  dgrp_t group, void * priv_data);
+static void adjust_extattr_refcount(e2fsck_t ctx, ext2_refcount_t refcount, 
+				    char *block_buf, int adjust_sign);
 /* static char *describe_illegal_block(ext2_filsys fs, blk_t block); */
 
 struct process_block_struct {
@@ -123,6 +127,11 @@ int e2fsck_pass1_check_device_inode(struct ext2_inode *inode)
 {
 	int	i;
 
+	/*
+	 * If i_blocks is non-zero, then this is a bogus device/fifo/socket
+	 */
+	if (inode->i_blocks)
+		return 0;
 	/*
 	 * We should be able to do the test below all the time, but
 	 * because the kernel doesn't forcibly clear the device
@@ -142,6 +151,55 @@ int e2fsck_pass1_check_device_inode(struct ext2_inode *inode)
 	return 1;
 }
 
+#ifndef HAVE_STRNLEN
+/*
+ * Incredibly, libc5 doesn't appear to have strnlen.  So we have to
+ * provide our own.
+ */
+static int strnlen(const char * s, int count)
+{
+	const char *cp = s;
+
+	while (count-- && *cp)
+		cp++;
+	return cp - s;
+}
+#endif
+
+/*
+ * Check to make sure a symlink inode is real.  Returns 1 if the symlink
+ * checks out, 0 if not.
+ */
+int e2fsck_pass1_check_symlink(ext2_filsys fs, struct ext2_inode *inode)
+{
+	int i;
+
+	if ((inode->i_size_high || inode->i_size == 0) ||
+	    (inode->i_flags & (EXT2_IMMUTABLE_FL | EXT2_APPEND_FL |
+			       EXT2_INDEX_FL)))
+		return 0;
+
+	if (inode->i_blocks) {
+		if ((inode->i_size > fs->blocksize) ||
+		    (inode->i_blocks != fs->blocksize >> 9) ||
+		    (inode->i_block[0] < fs->super->s_first_data_block) ||
+		    (inode->i_block[0] >= fs->super->s_blocks_count))
+			return 0;
+
+		for (i = 1; i < EXT2_N_BLOCKS; i++)
+			if (inode->i_block[i])
+				return 0;
+	} else {
+		if (inode->i_size > sizeof(inode->i_block) - 1)
+			return 0;
+
+		if (inode->i_size !=
+		    strnlen((char *)inode->i_block, sizeof(inode->i_block)))
+			return 0;
+	}
+	return 1;
+}
+
 /*
  * If the immutable (or append-only) flag is set on the inode, offer
  * to clear it.
@@ -166,14 +224,10 @@ static void check_size(e2fsck_t ctx, struct problem_context *pctx)
 {
 	struct ext2_inode *inode = pctx->inode;
 	
-	if ((LINUX_S_ISBLK(inode->i_mode) ||
-	     LINUX_S_ISCHR(inode->i_mode) ||
-	     LINUX_S_ISFIFO(inode->i_mode) ||
-	     LINUX_S_ISSOCK(inode->i_mode)) && 
-	    !inode->i_size || !inode->i_size_high)
+	if ((inode->i_size == 0) && (inode->i_size_high == 0))
 		return;
 	
-	if(!fix_problem(ctx, PR_1_SET_NONZSIZE, pctx))
+	if (!fix_problem(ctx, PR_1_SET_NONZSIZE, pctx))
 		return;
 	
 	inode->i_size = 0;
@@ -428,6 +482,10 @@ void e2fsck_pass1(e2fsck_t ctx)
 			if (ino == EXT2_BOOT_LOADER_INO) {
 				if (LINUX_S_ISDIR(inode.i_mode))
 					problem = PR_1_RESERVED_BAD_MODE;
+			} else if (ino == EXT2_RESIZE_INO) {
+				if (inode.i_mode &&
+				    !LINUX_S_ISREG(inode.i_mode))
+					problem = PR_1_RESERVED_BAD_MODE;
 			} else {
 				if (inode.i_mode != 0)
 					problem = PR_1_RESERVED_BAD_MODE;
@@ -519,8 +577,7 @@ void e2fsck_pass1(e2fsck_t ctx)
 			frag = fsize = 0;
 		}
 		
-		if (inode.i_faddr || frag || fsize
-		    || inode.i_file_acl ||
+		if (inode.i_faddr || frag || fsize ||
 		    (LINUX_S_ISDIR(inode.i_mode) && inode.i_dir_acl))
 			mark_inode_bad(ctx, ino);
 		if (inode.i_flags & EXT2_IMAGIC_FL) {
@@ -556,11 +613,11 @@ void e2fsck_pass1(e2fsck_t ctx)
 			check_size(ctx, &pctx);
 			ctx->fs_blockdev_count++;
 		} else if (LINUX_S_ISLNK (inode.i_mode)) {
+			check_immutable(ctx, &pctx);
 			ctx->fs_symlinks_count++;
+			if (!e2fsck_pass1_check_symlink(fs, &inode))
+				mark_inode_bad(ctx, ino);
 			if (!inode.i_blocks) {
-				if (inode.i_size_high ||
-				    (inode.i_size > EXT2_N_BLOCKS*4))
-					mark_inode_bad(ctx, ino);
 				ctx->fs_fast_symlinks_count++;
 				goto next;
 			}
@@ -585,7 +642,8 @@ void e2fsck_pass1(e2fsck_t ctx)
 			ctx->fs_tind_count++;
 		if (inode.i_block[EXT2_IND_BLOCK] ||
 		    inode.i_block[EXT2_DIND_BLOCK] ||
-		    inode.i_block[EXT2_TIND_BLOCK]) {
+		    inode.i_block[EXT2_TIND_BLOCK] ||
+		    inode.i_file_acl) {
 			inodes_to_process[process_inode_count].ino = ino;
 			inodes_to_process[process_inode_count].inode = inode;
 			process_inode_count++;
@@ -622,9 +680,32 @@ void e2fsck_pass1(e2fsck_t ctx)
 	ext2fs_close_inode_scan(scan);
 	ehandler_operation(0);
 
+	/*
+	 * If any extended attribute blocks' reference counts need to
+	 * be adjusted, either up (ctx->refcount_extra), or down
+	 * (ctx->refcount), then fix them.
+	 */
+	if (ctx->refcount) {
+		adjust_extattr_refcount(ctx, ctx->refcount, block_buf, -1);
+		ea_refcount_free(ctx->refcount);
+		ctx->refcount = 0;
+	}
+	if (ctx->refcount_extra) {
+		adjust_extattr_refcount(ctx, ctx->refcount_extra,
+					block_buf, +1);
+		ea_refcount_free(ctx->refcount_extra);
+		ctx->refcount_extra = 0;
+	}
+		
 	if (ctx->invalid_bitmaps)
 		handle_fs_bad_blocks(ctx);
 
+	/* We don't need the block_ea_map any more */
+	if (ctx->block_ea_map) {
+		ext2fs_free_block_bitmap(ctx->block_ea_map);
+		ctx->block_ea_map = 0;
+	}
+
 	if (ctx->flags & E2F_FLAG_RESTART) {
 		/*
 		 * Only the master copy of the superblock and block
@@ -650,29 +731,6 @@ endit:
 	
 	ext2fs_free_mem((void **) &block_buf);
 
-	if (ctx->large_files) {
-		if (!(sb->s_feature_ro_compat &
-		      EXT2_FEATURE_RO_COMPAT_LARGE_FILE) &&
-		    fix_problem(ctx, PR_1_FEATURE_LARGE_FILES, &pctx)) {
-			sb->s_feature_ro_compat |=
-				EXT2_FEATURE_RO_COMPAT_LARGE_FILE;
-			ext2fs_mark_super_dirty(fs);
-		}
-		if (sb->s_rev_level == EXT2_GOOD_OLD_REV &&
-		    fix_problem(ctx, PR_1_FS_REV_LEVEL, &pctx)) {
-			ext2fs_update_dynamic_rev(fs);
-			ext2fs_mark_super_dirty(fs);
-		}
-	} else if (!ctx->large_files &&
-	    (sb->s_feature_ro_compat &
-	      EXT2_FEATURE_RO_COMPAT_LARGE_FILE)) {
-		if (fs->flags & EXT2_FLAG_RW) {
-			sb->s_feature_ro_compat &= 
-				~EXT2_FEATURE_RO_COMPAT_LARGE_FILE;
-			ext2fs_mark_super_dirty(fs);
-		}
-	}
-	
 #ifdef RESOURCE_TRACK
 	if (ctx->options & E2F_OPT_TIME2) {
 		e2fsck_clear_progbar(ctx);
@@ -756,9 +814,13 @@ static EXT2_QSORT_TYPE process_inode_cmp(const void *a, const void *b)
 		(const struct process_inode_block *) a;
 	const struct process_inode_block *ib_b =
 		(const struct process_inode_block *) b;
-
-	return (ib_a->inode.i_block[EXT2_IND_BLOCK] -
-		ib_b->inode.i_block[EXT2_IND_BLOCK]);
+	int	ret;
+	
+	ret = (ib_a->inode.i_block[EXT2_IND_BLOCK] -
+	       ib_b->inode.i_block[EXT2_IND_BLOCK]);
+	if (ret == 0)
+		ret = ib_a->inode.i_file_acl - ib_b->inode.i_file_acl;
+	return ret;
 }
 
 /*
@@ -858,6 +920,208 @@ static _INLINE_ void mark_block_used(e2fsck_t ctx, blk_t block)
 	}
 }
 
+/*
+ * Adjust the extended attribute block's reference counts at the end
+ * of pass 1, either by subtracting out references for EA blocks that
+ * are still referenced in ctx->refcount, or by adding references for
+ * EA blocks that had extra references as accounted for in
+ * ctx->refcount_extra.
+ */
+static void adjust_extattr_refcount(e2fsck_t ctx, ext2_refcount_t refcount, 
+				    char *block_buf, int adjust_sign)
+{
+	struct ext2_ext_attr_header 	*header;
+	struct problem_context		pctx;
+	ext2_filsys			fs = ctx->fs;
+	blk_t				blk;
+	__u32				should_be;
+	int				count;
+
+	clear_problem_context(&pctx);
+	
+	ea_refcount_intr_begin(refcount);
+	while (1) {
+		if ((blk = ea_refcount_intr_next(refcount, &count)) == 0)
+			break;
+		pctx.blk = blk;
+		pctx.errcode = ext2fs_read_ext_attr(fs, blk, block_buf);
+		if (pctx.errcode) {
+			fix_problem(ctx, PR_1_EXTATTR_READ_ABORT, &pctx);
+			return;
+		}
+		header = (struct ext2_ext_attr_header *) block_buf;
+		pctx.blkcount = header->h_refcount;
+		should_be = header->h_refcount + adjust_sign * count;
+		pctx.num = should_be;
+		if (fix_problem(ctx, PR_1_EXTATTR_REFCOUNT, &pctx)) {
+			header->h_refcount = should_be;
+			pctx.errcode = ext2fs_write_ext_attr(fs, blk,
+							     block_buf);
+			if (pctx.errcode) {
+				fix_problem(ctx, PR_1_EXTATTR_WRITE, &pctx);
+				continue;
+			}
+		}
+	}
+}
+
+/*
+ * Handle processing the extended attribute blocks
+ */
+static int check_ext_attr(e2fsck_t ctx, struct problem_context *pctx,
+			   char *block_buf)
+{
+	ext2_filsys fs = ctx->fs;
+	ext2_ino_t	ino = pctx->ino;
+	struct ext2_inode *inode = pctx->inode;
+	blk_t		blk;
+	char *		end;
+	struct ext2_ext_attr_header *header;
+	struct ext2_ext_attr_entry *entry;
+	int		count;
+	region_t	region;
+	
+	blk = inode->i_file_acl;
+	if (blk == 0)
+		return 0;
+
+	/*
+	 * If the Extended attribute flag isn't set, then a non-zero
+	 * file acl means that the inode is corrupted.
+	 *
+	 * Or if the extended attribute block is an invalid block,
+	 * then the inode is also corrupted.
+	 */
+	if (!(fs->super->s_feature_compat & EXT2_FEATURE_COMPAT_EXT_ATTR) ||
+	    (blk < fs->super->s_first_data_block) ||
+	    (blk >= fs->super->s_blocks_count)) {
+		mark_inode_bad(ctx, ino);
+		return 0;
+	}
+
+	/* If ea bitmap hasn't been allocated, create it */
+	if (!ctx->block_ea_map) {
+		pctx->errcode = ext2fs_allocate_block_bitmap(fs,
+						      _("ext attr block map"),
+						      &ctx->block_ea_map);
+		if (pctx->errcode) {
+			pctx->num = 2;
+			fix_problem(ctx, PR_1_ALLOCATE_BBITMAP_ERROR, pctx);
+			ctx->flags |= E2F_FLAG_ABORT;
+			return 0;
+		}
+	}
+
+	/* Create the EA refcount structure if necessary */
+	if (!ctx->refcount) {
+		pctx->errcode = ea_refcount_create(0, &ctx->refcount);
+		if (pctx->errcode) {
+			pctx->num = 1;
+			fix_problem(ctx, PR_1_ALLOCATE_REFCOUNT, pctx);
+			ctx->flags |= E2F_FLAG_ABORT;
+			return 0;
+		}
+	}
+
+#if 0
+	/* Debugging text */
+	printf("Inode %u has EA block %u\n", ino, blk);
+#endif
+
+	/* Have we seen this EA block before? */
+	if (ext2fs_fast_test_block_bitmap(ctx->block_ea_map, blk)) {
+		if (ea_refcount_decrement(ctx->refcount, blk, 0) == 0)
+			return 1;
+		/* Ooops, this EA was referenced more than it stated */
+		if (!ctx->refcount_extra) {
+			pctx->errcode = ea_refcount_create(0,
+					   &ctx->refcount_extra);
+			if (pctx->errcode) {
+				pctx->num = 2;
+				fix_problem(ctx, PR_1_ALLOCATE_REFCOUNT, pctx);
+				ctx->flags |= E2F_FLAG_ABORT;
+				return 0;
+			}
+		}
+		ea_refcount_increment(ctx->refcount_extra, blk, 0);
+		return 1;
+	}
+	
+	/*
+	 * OK, we haven't seen this EA block yet.  So we need to
+	 * validate it
+	 */
+	pctx->blk = blk;
+	pctx->errcode = ext2fs_read_ext_attr(fs, blk, block_buf);
+	if (pctx->errcode && fix_problem(ctx, PR_1_READ_EA_BLOCK, pctx))
+		goto clear_extattr;
+	header = (struct ext2_ext_attr_header *) block_buf;
+	pctx->blk = inode->i_file_acl;
+	if (header->h_magic != EXT2_EXT_ATTR_MAGIC) {
+		if (fix_problem(ctx, PR_1_BAD_EA_BLOCK, pctx))
+			goto clear_extattr;
+	}
+	if (header->h_blocks != 1) {
+		if (fix_problem(ctx, PR_1_EA_MULTI_BLOCK, pctx))
+			goto clear_extattr;
+	}
+
+	region = region_create(0, fs->blocksize);
+	if (!region) {
+		fix_problem(ctx, PR_1_EA_ALLOC_REGION, pctx);
+		ctx->flags |= E2F_FLAG_ABORT;
+		return 0;
+	}
+	if (region_allocate(region, 0, sizeof(struct ext2_ext_attr_header))) {
+		if (fix_problem(ctx, PR_1_EA_ALLOC_COLLISION, pctx))
+			goto clear_extattr;
+	}
+	
+	entry = (struct ext2_ext_attr_entry *)(header+1);
+	end = block_buf + fs->blocksize;
+	while ((char *)entry < end && *(__u32 *)entry) {
+		if (region_allocate(region, (char *)entry - (char *)header,
+			           EXT2_EXT_ATTR_LEN(entry->e_name_len))) {
+			if (fix_problem(ctx, PR_1_EA_ALLOC_COLLISION, pctx))
+				goto clear_extattr;
+		}
+		if (entry->e_name_len == 0 || entry->e_name_index != 0) {
+			if (fix_problem(ctx, PR_1_EA_BAD_NAME, pctx))
+				goto clear_extattr;
+		}
+		if (entry->e_value_block != 0) {
+			if (fix_problem(ctx, PR_1_EA_BAD_VALUE, pctx))
+				goto clear_extattr;
+		}
+		if (entry->e_value_size &&
+		    region_allocate(region, entry->e_value_offs,
+				    EXT2_EXT_ATTR_SIZE(entry->e_value_size))) {
+			if (fix_problem(ctx, PR_1_EA_ALLOC_COLLISION, pctx))
+				goto clear_extattr;
+		}
+		entry = EXT2_EXT_ATTR_NEXT(entry);
+	}
+	if (region_allocate(region, (char *)entry - (char *)header, 4)) {
+		if (fix_problem(ctx, PR_1_EA_ALLOC_COLLISION, pctx))
+			goto clear_extattr;
+	}
+	region_free(region);
+
+	count = header->h_refcount - 1;
+	if (count)
+		ea_refcount_store(ctx->refcount, blk, count);
+	mark_block_used(ctx, blk);
+	ext2fs_fast_mark_block_bitmap(ctx->block_ea_map, blk);
+	
+	return 1;
+
+clear_extattr:
+	inode->i_file_acl = 0;
+	e2fsck_write_inode(ctx, ino, inode, "check_ext_attr");
+	return 0;
+}
+
+
 /*
  * This subroutine is called on each inode to account for all of the
  * blocks used by that inode.
@@ -930,6 +1194,8 @@ static void check_blocks(e2fsck_t ctx, struct problem_context *pctx,
 		ctx->flags |= E2F_FLAG_RESTART;
 		return;
 	}
+	if (inode->i_file_acl && check_ext_attr(ctx, pctx, block_buf))
+		pb.num_blocks++;
 
 	pb.num_blocks *= (fs->blocksize / 512);
 #if 0
