@@ -37,6 +37,9 @@
 
 #include "problem.h"
 
+/* Define an extension to the ext2 library's block count information */
+#define BLOCK_COUNT_EXTATTR	(-5)
+
 /*
  * This is structure is allocated for each time that a block is
  * claimed by more than one file.  So if a particular block is claimed
@@ -67,12 +70,15 @@ struct dup_block {
 	blk_t		block;		/* Block number */
 	ext2_ino_t	ino;		/* Inode number */
 	int		num_bad;
+	int		flags;
 	/* Pointer to next dup record with different block */
 	struct dup_block *next_block;
 	/* Pointer to next dup record with different inode */
 	struct dup_block *next_inode;
 };
 
+#define FLAG_EXTATTR	(1)
+
 /*
  * This structure stores information about a particular inode which
  * is sharing blocks with other inodes.  This information is collected
@@ -166,8 +172,10 @@ static void pass1b(e2fsck_t ctx, char *block_buf)
 	ext2_inode_scan	scan;
 	struct process_block_struct pb;
 	struct dup_inode *dp;
+	struct dup_block *q, *r;
 	struct problem_context pctx;
-
+	int	i, ea_flag;
+	
 	clear_problem_context(&pctx);
 	
 	fix_problem(ctx, PR_1B_PASS_HEADER, &pctx);
@@ -199,6 +207,9 @@ static void pass1b(e2fsck_t ctx, char *block_buf)
 		pb.dup_blocks = 0;
 		pctx.errcode = ext2fs_block_iterate2(fs, ino, 0, block_buf,
 					      process_pass1b_block, &pb);
+		if (inode.i_file_acl)
+			process_pass1b_block(fs, &inode.i_file_acl,
+					     BLOCK_COUNT_EXTATTR, 0, 0, &pb);
 		if (pb.dup_blocks) {
 			end_problem_latch(ctx, PR_LATCH_DBLOCK);
 			dp = (struct dup_inode *) e2fsck_allocate_memory(ctx,
@@ -227,6 +238,21 @@ static void pass1b(e2fsck_t ctx, char *block_buf)
 	}
 	ext2fs_close_inode_scan(scan);
 	e2fsck_use_inode_shortcuts(ctx, 0);
+	/*
+	 * Set the num_bad field
+	 */
+	for (q = dup_blk; q; q = q->next_block) {
+		i = 0;
+		ea_flag = 0;
+		for (r = q; r; r = r->next_inode) {
+			if (r->flags & FLAG_EXTATTR) {
+				if (ea_flag++)
+					continue;
+			}
+			i++;
+		}
+		q->num_bad = i;
+	}
 }
 
 static int process_pass1b_block(ext2_filsys fs,
@@ -237,8 +263,7 @@ static int process_pass1b_block(ext2_filsys fs,
 				void *priv_data)
 {
 	struct process_block_struct *p;
-	struct dup_block *dp, *q, *r;
-	int i;
+	struct dup_block *dp, *q;
 	e2fsck_t ctx;
 
 	if (HOLE_BLKADDR(*block_nr))
@@ -261,6 +286,8 @@ static int process_pass1b_block(ext2_filsys fs,
 		dp->block = *block_nr;
 		dp->ino = p->ino;
 		dp->num_bad = 0;
+		dp->flags = (blockcnt == BLOCK_COUNT_EXTATTR) ?
+			FLAG_EXTATTR : 0;
 		q = dup_blk;
 		while (q) {
 			if (q->block == *block_nr)
@@ -275,15 +302,6 @@ static int process_pass1b_block(ext2_filsys fs,
 			dup_blk = dp;
 		}
 	}
-	/*
-	 * Set the num_bad field
-	 */
-	for (q = dup_blk; q; q = q->next_block) {
-		i = 0;
-		for (r = q; r; r = r->next_inode)
-			i++;
-		q->num_bad = i;
-	}
 	return 0;
 }
 
@@ -481,6 +499,18 @@ static void pass1d(e2fsck_t ctx, char *block_buf)
 	ext2fs_free_mem((void **) &shared);
 }
 
+/*
+ * Drop the refcount on the dup_block structure, and clear the entry
+ * in the block_dup_map if appropriate.
+ */
+static void decrement_badcount(e2fsck_t ctx, struct dup_block *p)
+{
+	p->num_bad--;
+	if (p->num_bad <= 0 ||
+	    (p->num_bad == 1 && !check_if_fs_block(ctx, p->block)))
+		ext2fs_unmark_block_bitmap(ctx->block_dup_map, p->block);
+}
+
 static int delete_file_block(ext2_filsys fs,
 			     blk_t	*block_nr,
 			     e2_blkcnt_t blockcnt,
@@ -503,10 +533,7 @@ static int delete_file_block(ext2_filsys fs,
 			if (p->block == *block_nr)
 				break;
 		if (p) {
-			p->num_bad--;
-			if (p->num_bad == 1)
-				ext2fs_unmark_block_bitmap(ctx->block_dup_map,
-							   *block_nr);
+			decrement_badcount(ctx, p);
 		} else
 			com_err("delete_file_block", 0,
 			    _("internal error; can't find dup_blk for %d\n"),
@@ -546,6 +573,9 @@ static void delete_file(e2fsck_t ctx, struct dup_inode *dp, char* block_buf)
 	e2fsck_read_inode(ctx, dp->ino, &inode, "delete_file");
 	inode.i_links_count = 0;
 	inode.i_dtime = time(0);
+	if (inode.i_file_acl)
+		delete_file_block(fs, &inode.i_file_acl,
+				  BLOCK_COUNT_EXTATTR, 0, 0, &pb);
 	e2fsck_write_inode(ctx, dp->ino, &inode, "delete_file");
 }
 
@@ -593,6 +623,10 @@ static int clone_file_block(ext2_filsys fs,
 					return BLOCK_ABORT;
 				}
 			}
+#if 0
+			printf("Cloning block %u to %u\n", *block_nr,
+			       new_block);
+#endif
 			retval = io_channel_read_blk(fs->io, *block_nr, 1,
 						     cs->buf);
 			if (retval) {
@@ -605,11 +639,7 @@ static int clone_file_block(ext2_filsys fs,
 				cs->errcode = retval;
 				return BLOCK_ABORT;
 			}
-			p->num_bad--;
-			if (p->num_bad == 1 &&
-			    !check_if_fs_block(ctx, *block_nr))
-				ext2fs_unmark_block_bitmap(ctx->block_dup_map,
-							   *block_nr);
+			decrement_badcount(ctx, p);
 			*block_nr = new_block;
 			ext2fs_mark_block_bitmap(ctx->block_found_map,
 						 new_block);
@@ -629,6 +659,7 @@ static int clone_file(e2fsck_t ctx, struct dup_inode *dp, char* block_buf)
 	errcode_t	retval;
 	struct clone_struct cs;
 	struct problem_context	pctx;
+	blk_t		blk;
 
 	clear_problem_context(&pctx);
 	cs.errcode = 0;
@@ -646,17 +677,52 @@ static int clone_file(e2fsck_t ctx, struct dup_inode *dp, char* block_buf)
 	pctx.errcode = ext2fs_block_iterate2(fs, dp->ino, 0, block_buf,
 				      clone_file_block, &cs);
 	ext2fs_mark_bb_dirty(fs);
-	ext2fs_free_mem((void **) &cs.buf);
 	if (pctx.errcode) {
 		fix_problem(ctx, PR_1B_BLOCK_ITERATE, &pctx);
-		return pctx.errcode;
+		retval = pctx.errcode;
+		goto errout;
 	}
 	if (cs.errcode) {
 		com_err("clone_file", cs.errcode,
 			_("returned from clone_file_block"));
-		return cs.errcode;
+		retval = cs.errcode;
+		goto errout;
 	}
-	return 0;
+	blk = dp->inode.i_file_acl;
+	if (blk && (clone_file_block(fs, &dp->inode.i_file_acl,
+				     BLOCK_COUNT_EXTATTR, 0, 0, &cs) ==
+		    BLOCK_CHANGED)) {
+		struct dup_block *p, *q;
+		struct dup_inode *r;
+
+		/*
+		 * If we cloned the EA block, find all other inodes
+		 * which refered to that EA block, and modify
+		 * them to point to the new EA block.
+		 */
+		for (p = dup_blk; p; p = p->next_block) {
+			if (p->block == blk)
+				break;
+		}
+		for (q = p; q ; q = q->next_inode) {
+			if (!(q->flags & FLAG_EXTATTR))
+				continue;
+			for (r = dup_ino; r; r = r->next)
+				if (r->ino == q->ino)
+					break;
+			if (r) {
+				r->inode.i_file_acl = dp->inode.i_file_acl;
+				e2fsck_write_inode(ctx, q->ino, &r->inode,
+						   "clone file EA");
+			}
+			q->ino = 0; /* Should free the structure... */
+			decrement_badcount(ctx, p);
+		}
+	}
+	retval = 0;
+errout:
+	ext2fs_free_mem((void **) &cs.buf);
+	return retval;
 }
 
 /*
