@@ -76,12 +76,14 @@ struct check_dir_struct {
 
 void e2fsck_pass2(e2fsck_t ctx)
 {
-	ext2_filsys 	fs = ctx->fs;
-	char	*buf;
+	struct ext2_super_block *sb = ctx->fs->super;
+	struct problem_context	pctx;
+	ext2_filsys 		fs = ctx->fs;
+	char			*buf;
 #ifdef RESOURCE_TRACK
 	struct resource_track	rtrack;
 #endif
-	struct dir_info *dir;
+	struct dir_info 	*dir;
 	struct check_dir_struct cd;
 		
 #ifdef RESOURCE_TRACK
@@ -146,6 +148,31 @@ void e2fsck_pass2(e2fsck_t ctx)
 		ext2fs_free_inode_bitmap(ctx->inode_reg_map);
 		ctx->inode_reg_map = 0;
 	}
+
+	clear_problem_context(&pctx);
+	if (ctx->large_files) {
+		if (!(sb->s_feature_ro_compat &
+		      EXT2_FEATURE_RO_COMPAT_LARGE_FILE) &&
+		    fix_problem(ctx, PR_2_FEATURE_LARGE_FILES, &pctx)) {
+			sb->s_feature_ro_compat |=
+				EXT2_FEATURE_RO_COMPAT_LARGE_FILE;
+			ext2fs_mark_super_dirty(fs);
+		}
+		if (sb->s_rev_level == EXT2_GOOD_OLD_REV &&
+		    fix_problem(ctx, PR_1_FS_REV_LEVEL, &pctx)) {
+			ext2fs_update_dynamic_rev(fs);
+			ext2fs_mark_super_dirty(fs);
+		}
+	} else if (!ctx->large_files &&
+	    (sb->s_feature_ro_compat &
+	      EXT2_FEATURE_RO_COMPAT_LARGE_FILE)) {
+		if (fs->flags & EXT2_FLAG_RW) {
+			sb->s_feature_ro_compat &= 
+				~EXT2_FEATURE_RO_COMPAT_LARGE_FILE;
+			ext2fs_mark_super_dirty(fs);
+		}
+	}
+	
 #ifdef RESOURCE_TRACK
 	if (ctx->options & E2F_OPT_TIME2) {
 		e2fsck_clear_progbar(ctx);
@@ -647,7 +674,17 @@ static void deallocate_inode(e2fsck_t ctx, ext2_ino_t ino, char* block_buf)
 
 	if (!ext2fs_inode_has_valid_blocks(&inode))
 		return;
-	
+
+	if (!LINUX_S_ISDIR(inode.i_mode) &&
+	    (inode.i_size_high || inode.i_size & 0x80000000UL))
+		ctx->large_files--;
+
+	if (inode.i_file_acl) {
+		ext2fs_unmark_block_bitmap(ctx->block_found_map,
+					   inode.i_file_acl);
+		ext2fs_unmark_block_bitmap(fs->block_map, inode.i_file_acl);
+	}
+
 	ext2fs_mark_bb_dirty(fs);
 	pctx.errcode = ext2fs_block_iterate2(fs, ino, 0, block_buf,
 					    deallocate_inode_block, ctx);
@@ -693,9 +730,9 @@ extern int e2fsck_process_bad_inode(e2fsck_t ctx, ext2_ino_t dir,
 		 && !e2fsck_pass1_check_device_inode(&inode))
 		problem = PR_2_BAD_SOCKET;
 	else if (LINUX_S_ISLNK(inode.i_mode)
-		 && (inode.i_size_high ||
-		     (inode.i_size > EXT2_N_BLOCKS*4)))
+		 && !e2fsck_pass1_check_symlink(fs, &inode)) {
 		problem = PR_2_SYMLINK_SIZE;
+	}
 
 	if (problem) {
 		if (fix_problem(ctx, problem, &pctx)) {
@@ -747,10 +784,18 @@ extern int e2fsck_process_bad_inode(e2fsck_t ctx, ext2_ino_t dir,
 	}
 
 	if (inode.i_file_acl &&
+	    !(fs->super->s_feature_compat & EXT2_FEATURE_COMPAT_EXT_ATTR) &&
 	    fix_problem(ctx, PR_2_FILE_ACL_ZERO, &pctx)) {
 		inode.i_file_acl = 0;
 		inode_modified++;
 	}
+	if (inode.i_file_acl &&
+	    ((inode.i_file_acl < fs->super->s_first_data_block) ||
+	     (inode.i_file_acl >= fs->super->s_blocks_count)) &&
+	    fix_problem(ctx, PR_2_FILE_ACL_BAD, &pctx)) {
+		inode.i_file_acl = 0;
+		inode_modified++;
+	}
 	if (inode.i_dir_acl &&
 	    LINUX_S_ISDIR(inode.i_mode) &&
 	    fix_problem(ctx, PR_2_DIR_ACL_ZERO, &pctx)) {
