@@ -42,14 +42,14 @@
 #include "problem.h"
 
 static void check_root(e2fsck_t ctx);
-static void check_directory(e2fsck_t ctx, struct dir_info *dir,
-			    struct problem_context *pctx);
-static ino_t get_lost_and_found(e2fsck_t ctx);
-static void fix_dotdot(e2fsck_t ctx, struct dir_info *dir, ino_t parent);
-static errcode_t adjust_inode_count(e2fsck_t ctx, ino_t ino, int adj);
-static errcode_t expand_directory(e2fsck_t ctx, ino_t dir);
+static int check_directory(e2fsck_t ctx, struct dir_info *dir,
+			   struct problem_context *pctx);
+static ext2_ino_t get_lost_and_found(e2fsck_t ctx);
+static void fix_dotdot(e2fsck_t ctx, struct dir_info *dir, ext2_ino_t parent);
+static errcode_t adjust_inode_count(e2fsck_t ctx, ext2_ino_t ino, int adj);
+static errcode_t expand_directory(e2fsck_t ctx, ext2_ino_t dir);
 
-static ino_t lost_and_found = 0;
+static ext2_ino_t lost_and_found = 0;
 static int bad_lost_and_found = 0;
 
 static ext2fs_inode_bitmap inode_loop_detect = 0;
@@ -82,15 +82,7 @@ void e2fsck_pass3(e2fsck_t ctx)
 	/*
 	 * Allocate some bitmaps to do loop detection.
 	 */
-	pctx.errcode = ext2fs_allocate_inode_bitmap(fs,
-		    "inode loop detection bitmap", &inode_loop_detect);
-	if (pctx.errcode) {
-		pctx.num = 1;
-		fix_problem(ctx, PR_3_ALLOCATE_IBITMAP_ERROR, &pctx);
-		ctx->flags |= E2F_FLAG_ABORT;
-		goto abort_exit;
-	}
-	pctx.errcode = ext2fs_allocate_inode_bitmap(fs, "inode done bitmap",
+	pctx.errcode = ext2fs_allocate_inode_bitmap(fs, _("inode done bitmap"),
 						    &inode_done_map);
 	if (pctx.errcode) {
 		pctx.num = 2;
@@ -101,7 +93,7 @@ void e2fsck_pass3(e2fsck_t ctx)
 #ifdef RESOURCE_TRACK
 	if (ctx->options & E2F_OPT_TIME) {
 		e2fsck_clear_progbar(ctx);
-		print_resource_track("Peak memory", &ctx->global_rtrack);
+		print_resource_track(_("Peak memory"), &ctx->global_rtrack);
 	}
 #endif
 
@@ -123,7 +115,8 @@ void e2fsck_pass3(e2fsck_t ctx)
 			if ((ctx->progress)(ctx, 3, count++, maxdirs))
 				goto abort_exit;
 		if (ext2fs_test_inode_bitmap(ctx->inode_dir_map, dir->ino))
-			check_directory(ctx, dir, &pctx);
+			if (check_directory(ctx, dir, &pctx))
+				goto abort_exit;
 	}
 
 	/*
@@ -134,14 +127,18 @@ void e2fsck_pass3(e2fsck_t ctx)
 
 abort_exit:
 	e2fsck_free_dir_info(ctx);
-	if (inode_loop_detect)
+	if (inode_loop_detect) {
 		ext2fs_free_inode_bitmap(inode_loop_detect);
-	if (inode_done_map)
+		inode_loop_detect = 0;
+	}
+	if (inode_done_map) {
 		ext2fs_free_inode_bitmap(inode_done_map);
+		inode_done_map = 0;
+	}
 #ifdef RESOURCE_TRACK
 	if (ctx->options & E2F_OPT_TIME2) {
 		e2fsck_clear_progbar(ctx);
-		print_resource_track("Pass 3", &rtrack);
+		print_resource_track(_("Pass 3"), &rtrack);
 	}
 #endif
 }
@@ -259,34 +256,38 @@ static void check_root(e2fsck_t ctx)
  * the lost+found.  We have to do loop detection; if we ever discover
  * a loop, we treat that as a disconnected directory and offer to
  * reparent it to lost+found.
+ * 
+ * However, loop detection is expensive, because for very large
+ * filesystems, the inode_loop_detect bitmap is huge, and clearing it
+ * is non-trivial.  Loops in filesystems are also a rare error case,
+ * and we shouldn't optimize for error cases.  So we try two passes of
+ * the algorithm.  The first time, we ignore loop detection and merely
+ * increment a counter; if the counter exceeds some extreme threshold,
+ * then we try again with the loop detection bitmap enabled.
  */
-static void check_directory(e2fsck_t ctx, struct dir_info *dir,
-			    struct problem_context *pctx)
+static int check_directory(e2fsck_t ctx, struct dir_info *dir,
+			   struct problem_context *pctx)
 {
-	ext2_filsys fs = ctx->fs;
+	ext2_filsys 	fs = ctx->fs;
 	struct dir_info *p = dir;
+	int		loop_pass = 0, parent_count = 0;
 
 	if (!p)
-		return;
+		return 0;
 
-	ext2fs_clear_inode_bitmap(inode_loop_detect);
-
-	/*
-	 * Keep going until we find a parent which we've already
-	 * checked.  We know it's either already connected to the
-	 * directory tree, or it isn't but the user has already told
-	 * us he doesn't want us to reconnect the disconnected
-	 * subtree.
-	 */
-	while (!ext2fs_test_inode_bitmap(inode_done_map, p->ino)) {
+	while (1) {
 		/*
 		 * Mark this inode as being "done"; by the time we
 		 * return from this function, the inode we either be
 		 * verified as being connected to the directory tree,
 		 * or we will have offered to reconnect this to
 		 * lost+found.
+		 *
+		 * If it was marked done already, then we've reached a
+		 * parent we've already checked.
 		 */
-		ext2fs_mark_inode_bitmap(inode_done_map, p->ino);
+	  	if (ext2fs_mark_inode_bitmap(inode_done_map, p->ino))
+			break;
 
 		/*
 		 * If this directory doesn't have a parent, or we've
@@ -294,8 +295,9 @@ static void check_directory(e2fsck_t ctx, struct dir_info *dir,
 		 * reparent it to lost+found
 		 */
 		if (!p->parent ||
-		    (ext2fs_test_inode_bitmap(inode_loop_detect,
-					      p->parent))) {
+		    (loop_pass && 
+		     (ext2fs_test_inode_bitmap(inode_loop_detect,
+					      p->parent)))) {
 			pctx->ino = p->ino;
 			if (fix_problem(ctx, PR_3_UNCONNECTED_DIR, pctx)) {
 				if (e2fsck_reconnect_file(ctx, p->ino))
@@ -307,13 +309,35 @@ static void check_directory(e2fsck_t ctx, struct dir_info *dir,
 			}
 			break;
 		}
-		ext2fs_mark_inode_bitmap(inode_loop_detect,
-					 p->parent);
-		pctx->ino = p->parent;
 		p = e2fsck_get_dir_info(ctx, p->parent);
 		if (!p) {
 			fix_problem(ctx, PR_3_NO_DIRINFO, pctx);
-			return;
+			return 0;
+		}
+		if (loop_pass) {
+			ext2fs_mark_inode_bitmap(inode_loop_detect,
+						 p->ino);
+		} else if (parent_count++ > 2048) {
+			/*
+			 * If we've run into a path depth that's
+			 * greater than 2048, try again with the inode
+			 * loop bitmap turned on and start from the
+			 * top.
+			 */
+			loop_pass = 1;
+			if (inode_loop_detect)
+				ext2fs_clear_inode_bitmap(inode_loop_detect);
+			else {
+				pctx->errcode = ext2fs_allocate_inode_bitmap(fs, _("inode loop detection bitmap"), &inode_loop_detect);
+				if (pctx->errcode) {
+					pctx->num = 1;
+					fix_problem(ctx, 
+				    PR_3_ALLOCATE_IBITMAP_ERROR, pctx);
+					ctx->flags |= E2F_FLAG_ABORT;
+					return -1;
+				}
+			}
+			p = dir;
 		}
 	}
 
@@ -328,21 +352,22 @@ static void check_directory(e2fsck_t ctx, struct dir_info *dir,
 		if (fix_problem(ctx, PR_3_BAD_DOT_DOT, pctx))
 			fix_dotdot(ctx, dir, dir->parent);
 	}
+	return 0;
 }	
 
 /*
  * This routine gets the lost_and_found inode, making it a directory
  * if necessary
  */
-ino_t get_lost_and_found(e2fsck_t ctx)
+static ext2_ino_t get_lost_and_found(e2fsck_t ctx)
 {
 	ext2_filsys fs = ctx->fs;
-	ino_t			ino;
+	ext2_ino_t			ino;
 	blk_t			blk;
 	errcode_t		retval;
 	struct ext2_inode	inode;
 	char *			block;
-	const char 		name[] = "lost+found";
+	static const char	name[] = "lost+found";
 	struct 	problem_context	pctx;
 	struct dir_info 	*dirinfo;
 
@@ -462,7 +487,7 @@ ino_t get_lost_and_found(e2fsck_t ctx)
 	 * Miscellaneous bookkeeping that needs to be kept straight.
 	 */
 	e2fsck_add_dir_info(ctx, ino, EXT2_ROOT_INO);
-	adjust_inode_count(ctx, EXT2_ROOT_INO, +1);
+	adjust_inode_count(ctx, EXT2_ROOT_INO, 1);
 	ext2fs_icount_store(ctx->inode_count, ino, 2);
 	ext2fs_icount_store(ctx->inode_link_info, ino, 2);
 #if 0
@@ -474,7 +499,7 @@ ino_t get_lost_and_found(e2fsck_t ctx)
 /*
  * This routine will connect a file to lost+found
  */
-int e2fsck_reconnect_file(e2fsck_t ctx, ino_t ino)
+int e2fsck_reconnect_file(e2fsck_t ctx, ext2_ino_t ino)
 {
 	ext2_filsys fs = ctx->fs;
 	errcode_t	retval;
@@ -496,7 +521,7 @@ int e2fsck_reconnect_file(e2fsck_t ctx, ino_t ino)
 		return 1;
 	}
 	
-	sprintf(name, "#%lu", ino);
+	sprintf(name, "#%u", ino);
 	if (ext2fs_read_inode(fs, ino, &inode) == 0)
 		file_type = ext2_file_type(inode.i_mode);
 	retval = ext2fs_link(fs, lost_and_found, name, ino, file_type);
@@ -516,7 +541,7 @@ int e2fsck_reconnect_file(e2fsck_t ctx, ino_t ino)
 		fix_problem(ctx, PR_3_CANT_RECONNECT, &pctx);
 		return 1;
 	}
-	adjust_inode_count(ctx, ino, +1);
+	adjust_inode_count(ctx, ino, 1);
 
 	return 0;
 }
@@ -524,7 +549,7 @@ int e2fsck_reconnect_file(e2fsck_t ctx, ino_t ino)
 /*
  * Utility routine to adjust the inode counts on an inode.
  */
-static errcode_t adjust_inode_count(e2fsck_t ctx, ino_t ino, int adj)
+static errcode_t adjust_inode_count(e2fsck_t ctx, ext2_ino_t ino, int adj)
 {
 	ext2_filsys fs = ctx->fs;
 	errcode_t		retval;
@@ -556,9 +581,8 @@ static errcode_t adjust_inode_count(e2fsck_t ctx, ino_t ino, int adj)
 		inode.i_links_count--;
 	} else {
 		/* Should never happen */
-		printf("Debug error in e2fsck adjust_inode_count, "
-		       "should never happen.\n");
-		exit(1);
+		fatal_error(ctx, _("Debug error in e2fsck adjust_inode_count, "
+				   "should never happen.\n"));
 	}
 	
 	retval = ext2fs_write_inode(fs, ino, &inode);
@@ -573,7 +597,7 @@ static errcode_t adjust_inode_count(e2fsck_t ctx, ino_t ino, int adj)
  */
 struct fix_dotdot_struct {
 	ext2_filsys	fs;
-	ino_t		parent;
+	ext2_ino_t	parent;
 	int		done;
 	e2fsck_t	ctx;
 };
@@ -611,7 +635,7 @@ static int fix_dotdot_proc(struct ext2_dir_entry *dirent,
 	return DIRENT_ABORT | DIRENT_CHANGED;
 }
 
-static void fix_dotdot(e2fsck_t ctx, struct dir_info *dir, ino_t parent)
+static void fix_dotdot(e2fsck_t ctx, struct dir_info *dir, ext2_ino_t parent)
 {
 	ext2_filsys fs = ctx->fs;
 	errcode_t	retval;
@@ -656,7 +680,9 @@ struct expand_dir_struct {
 
 static int expand_dir_proc(ext2_filsys fs,
 			   blk_t	*blocknr,
-			   int	blockcnt,
+			   e2_blkcnt_t	blockcnt,
+			   blk_t ref_block,
+			   int ref_offset, 
 			   void	*priv_data)
 {
 	struct expand_dir_struct *es = (struct expand_dir_struct *) priv_data;
@@ -712,7 +738,7 @@ static int expand_dir_proc(ext2_filsys fs,
 		return BLOCK_CHANGED;
 }
 
-static errcode_t expand_directory(e2fsck_t ctx, ino_t dir)
+static errcode_t expand_directory(e2fsck_t ctx, ext2_ino_t dir)
 {
 	ext2_filsys fs = ctx->fs;
 	errcode_t	retval;
@@ -737,8 +763,8 @@ static errcode_t expand_directory(e2fsck_t ctx, ino_t dir)
 	es.newblocks = 0;
 	es.ctx = ctx;
 	
-	retval = ext2fs_block_iterate(fs, dir, BLOCK_FLAG_APPEND,
-				      0, expand_dir_proc, &es);
+	retval = ext2fs_block_iterate2(fs, dir, BLOCK_FLAG_APPEND,
+				       0, expand_dir_proc, &es);
 
 	if (es.err)
 		return es.err;
