@@ -38,6 +38,7 @@
 #define PROMPT_DELETE 	15
 #define PROMPT_SUPPRESS 16
 #define PROMPT_UNLINK	17
+#define PROMPT_NULL	18
 
 /*
  * These are the prompts which are used to ask the user if they want
@@ -62,6 +63,7 @@ static const char *prompt[] = {
 	N_("Delete file"),	/* 15 */
 	N_("Suppress messages"),/* 16 */
 	N_("Unlink"),		/* 17 */
+	"",			/* 18 */
 };
 
 /*
@@ -179,25 +181,25 @@ static const struct e2fsck_problem problem_table[] = {
 	  N_("@S has a bad ext3 @j (@i %i).\n"),
 	  PROMPT_CLEAR, PR_PREEN_OK },
 
-	/* Superblock has a journal device (which we can't handle yet) */
-	{ PR_0_JOURNAL_UNSUPP_DEV,
-	  N_("@S has external ext3 @j @v (unsupported).\n"),
-	  PROMPT_ABORT, PR_NO_OK | PR_AFTER_CODE, PR_0_JOURNAL_BAD_DEV },
+	/* The external journal has (unsupported) multiple filesystems */
+	{ PR_0_JOURNAL_UNSUPP_MULTIFS,
+	  N_("External @j has multiple @f users (unsupported).\n"),
+	  PROMPT_NONE, PR_FATAL },
 
-	/* Superblock has a bad journal device */
-	{ PR_0_JOURNAL_BAD_DEV,
-	  N_("@S has a bad ext3 @j (@v %X).\n"),
-	  PROMPT_CLEAR, PR_PREEN_OK },
+	/* Can't find external journal */
+	{ PR_0_CANT_FIND_JOURNAL,
+	  N_("Can't find external @j\n"),
+	  PROMPT_NONE, PR_FATAL },
 
-	/* Superblock has a journal UUID (which we can't handle yet) */
-	{ PR_0_JOURNAL_UNSUPP_UUID,
-	  N_("@S has an ext3 @j UUID (unsupported).\n"),
-	  PROMPT_ABORT, PR_NO_OK | PR_AFTER_CODE, PR_0_JOURNAL_BAD_UUID },
+	/* External journal has bad superblock */
+	{ PR_0_EXT_JOURNAL_BAD_SUPER,
+	  N_("External @j has bad @S\n"),
+	  PROMPT_NONE, PR_FATAL },
 
 	/* Superblock has a bad journal UUID */
 	{ PR_0_JOURNAL_BAD_UUID,
-	  N_("@S has a bad ext3 @j (UUID %s).\n"),
-	  PROMPT_CLEAR, PR_PREEN_OK },
+	  N_("External @j does not support this @f\n"),
+	  PROMPT_NONE, PR_FATAL },
 
 	/* Journal has an unknown superblock type */
 	{ PR_0_JOURNAL_UNSUPP_SUPER,
@@ -214,7 +216,7 @@ static const struct e2fsck_problem problem_table[] = {
 
 	/* Superblock flag should be cleared */
 	{ PR_0_JOURNAL_HAS_JOURNAL,
-	  N_("@S doesn't have has_journal flag, but has ext3 @j %s.\n"),
+	  N_("@S doesn't have has_@j flag, but has ext3 @j %s.\n"),
 	  PROMPT_CLEAR, PR_PREEN_OK },
 
 	/* Superblock flag is incorrect */
@@ -222,15 +224,25 @@ static const struct e2fsck_problem problem_table[] = {
 	  N_("@S has ext3 needs_recovery flag set, but no @j.\n"),
 	  PROMPT_CLEAR, PR_PREEN_OK },
 
-	/* Journal should be reset */
-	{ PR_0_JOURNAL_RESET_JOURNAL,
-	  N_("*** WARNING *** leaving data in the @j may be DANGEROUS.\n"),
-	  PROMPT_NONE, PR_PREEN_NOMSG|PR_AFTER_CODE, PR_0_JOURNAL_RESET_PROMPT},
+	/* Journal has data, but recovery flag is clear */
+	{ PR_0_JOURNAL_RECOVERY_CLEAR,
+	  N_("ext3 recovery flag clear, but @j has data.\n"),
+	  PROMPT_NONE, 0 },
 
-	/* Journal should be reset */
-	{ PR_0_JOURNAL_RESET_PROMPT,
-	  N_("ext3 recovery flag clear, but journal has data.\n"),
-	  PROMPT_CLEAR, PR_PREEN_OK|PR_PREEN_NOMSG },
+	/* Ask if we should clear the journal */
+	{ PR_0_JOURNAL_RESET_JOURNAL,
+	  N_("Clear @j"),
+	  PROMPT_NULL, PR_PREEN_NOMSG },
+
+	/* Ask if we should run the journal anyway */
+	{ PR_0_JOURNAL_RUN,
+	  N_("Run @j anyway"),
+	  PROMPT_NULL, 0 },
+
+	/* Run the journal by default */
+	{ PR_0_JOURNAL_RUN_DEFAULT,
+	  N_("Recovery flag not set in backup @S, so running @j anyway.\n"),
+	  PROMPT_NONE, 0 },
 
 	/* Clearing orphan inode */
 	{ PR_0_ORPHAN_CLEAR_INODE,
@@ -276,7 +288,23 @@ static const struct e2fsck_problem problem_table[] = {
 	{ PR_0_JOURNAL_UNSUPP_VERSION,
 	  N_("@j version not supported by this e2fsck.\n"),
 	  PROMPT_ABORT, 0 },
- 
+
+	/* Moving journal to hidden file */
+	{ PR_0_MOVE_JOURNAL,
+	  N_("Moving @j from /%s to hidden inode.\n\n"),
+	  PROMPT_NONE, 0 },
+
+	/* Error moving journal to hidden file */
+	{ PR_0_ERR_MOVE_JOURNAL,
+	  N_("Error moving @j: %m\n\n"),
+	  PROMPT_NONE, 0 },
+
+	/* Clearing V2 journal superblock */
+	{ PR_0_CLEAR_V2_JOURNAL,
+	  N_("Found invalid V2 @j @S fields (from V1 journal).\n"
+	     "Clearing fields beyond the V1 @j @S...\n\n"),
+	  PROMPT_NONE, 0 },
+
 	/* Pass 1 errors */
 	
 	/* Pass 1: Checking inodes, blocks, and sizes */
@@ -515,11 +543,6 @@ static const struct e2fsck_problem problem_table[] = {
 	/* Suppress messages prompt */
 	{ PR_1_SUPPRESS_MESSAGES, "", PROMPT_SUPPRESS, PR_NO_OK },
 		  
-	/* Filesystem contains large files, but has no such flag in sb */
-	{ PR_1_FEATURE_LARGE_FILES,
-	  N_("@f contains large files, but lacks LARGE_FILE flag in @S.\n"),
-	  PROMPT_FIX, 0 },
-	  
 	/* Imagic flag set on an inode when filesystem doesn't support it */
 	{ PR_1_SET_IMAGIC,
 	  N_("@i %i has imagic flag set.  "),
@@ -527,8 +550,8 @@ static const struct e2fsck_problem problem_table[] = {
 
 	/* Immutable flag set on a device or socket inode */
 	{ PR_1_SET_IMMUTABLE,
-	  N_("Special (@v/socket/fifo) @i %i has immutable or "
-	     "append-only flag set.\n"),
+	  N_("Special (@v/socket/fifo/symlink) file (@i %i) has immutable\n"
+	     "or append-only flag set.  "),
 	  PROMPT_CLEAR, PR_PREEN_OK | PR_PREEN_NO | PR_NO_OK },
 
 	/* Compression flag set on an inode when filesystem doesn't support it */
@@ -567,6 +590,61 @@ static const struct e2fsck_problem problem_table[] = {
 	  N_("@is that were part of a corrupted orphan linked list found.  "),
 	  PROMPT_FIX, 0 },		  
 		  
+	/* Error allocating refcount structure */
+	{ PR_1_ALLOCATE_REFCOUNT,
+	  "@A refcount structure (%N): %m\n",
+	  PROMPT_NONE, PR_FATAL },
+
+	/* Error reading extended attribute block */
+	{ PR_1_READ_EA_BLOCK,
+	  N_("Error reading @a @b %b for @i %i.  "),
+	  PROMPT_CLEAR, 0 },
+
+	/* Invalid extended attribute block */
+	{ PR_1_BAD_EA_BLOCK,
+	  N_("@i %i has a bad @a @b %b.  "),
+	  PROMPT_CLEAR, 0 },
+
+	/* Error reading Extended Attribute block while fixing refcount */
+	{ PR_1_EXTATTR_READ_ABORT,
+	  N_("Error reading @a @b %b (%m).  "),
+	  PROMPT_ABORT, 0 },
+		  
+	/* Extended attribute reference count incorrect */
+	{ PR_1_EXTATTR_REFCOUNT,
+	  N_("@a @b %b has reference count %B, should be %N.  "),
+	  PROMPT_FIX, 0 },
+		  
+	/* Error writing Extended Attribute block while fixing refcount */ 
+	{ PR_1_EXTATTR_WRITE,
+	  N_("Error writing @a @b %b (%m).  "),
+	  PROMPT_ABORT, 0 },
+
+	/* Multiple EA blocks not supported */
+	{ PR_1_EA_MULTI_BLOCK,
+	  N_("@a @b %b has h_blocks > 1.  "),
+	  PROMPT_CLEAR, 0},	  
+
+	/* Error allocating EA region allocation structure */
+	{ PR_1_EA_ALLOC_REGION,
+	  N_("Error allocating @a @b %b.  "),
+	  PROMPT_ABORT, 0},
+	
+	/* Error EA allocation collision */
+	{ PR_1_EA_ALLOC_COLLISION,
+	  N_("@a @b %b is corrupt (allocation collision).  "),
+	  PROMPT_CLEAR, 0},
+	
+	/* Bad extended attribute name */
+	{ PR_1_EA_BAD_NAME,
+	  N_("@a @b %b is corrupt (invalid name).  "),
+	  PROMPT_CLEAR, 0},	  
+
+	/* Bad extended attribute value */
+	{ PR_1_EA_BAD_VALUE,
+	  N_("@a @b %b is corrupt (invalid value).  "),
+	  PROMPT_CLEAR, 0},	  
+		  
 	/* Pass 1b errors */
 
 	/* Pass 1B: Rescan for duplicate/bad blocks */
@@ -803,7 +881,7 @@ static const struct e2fsck_problem problem_table[] = {
 
 	/* Final rec_len is wrong */
 	{ PR_2_FINAL_RECLEN,
-	  N_("@E has rec_len of %dr, should be %N.\n"),
+	  N_("@E has rec_len of %Dr, should be %N.\n"),
 	  PROMPT_FIX, 0 },
 		  
 	/* Error allocating icount structure */
@@ -858,24 +936,34 @@ static const struct e2fsck_problem problem_table[] = {
 
 	/* Directory filetype incorrect */
 	{ PR_2_BAD_FILETYPE,
-	  N_("@E has an incorrect filetype (was %dt, should be %N)\n"),
+	  N_("@E has an incorrect filetype (was %Dt, should be %N).\n"),
 	  PROMPT_FIX, 0 },
 
 	/* Directory filetype set on filesystem */
 	{ PR_2_CLEAR_FILETYPE,
-	  N_("@E has filetype set\n"),
+	  N_("@E has filetype set.\n"),
 	  PROMPT_CLEAR, PR_PREEN_OK },
 
 	/* Directory filename is null */
 	{ PR_2_NULL_NAME,
-	  N_("@E has a zero-length name\n"),
+	  N_("@E has a zero-length name.\n"),
 	  PROMPT_CLEAR, 0 },
 
 	/* Invalid fast symlink size */
 	{ PR_2_SYMLINK_SIZE,
-	  N_("@i %i (%Q) is a fast symlink with an invalid size (%Is)\n"),
+	  N_("Symlink %Q (@i #%i) has an invalid size (%Is).\n"),
 	  PROMPT_CLEAR, 0 },
 
+  	/* i_file_acl (extended attribute block) is bad */
+	{ PR_2_FILE_ACL_BAD,
+	  N_("@a @b @F is invalid (%If).\n"),
+	  PROMPT_CLEAR, 0 },
+
+	/* Filesystem contains large files, but has no such flag in sb */
+	{ PR_2_FEATURE_LARGE_FILES,
+	  N_("@f contains large files, but lacks LARGE_FILE flag in @S.\n"),
+	  PROMPT_FIX, 0 },
+	  
 	/* Pass 3 errors */
 
 	/* Pass 3: Checking directory connectivity */
@@ -910,7 +998,7 @@ static const struct e2fsck_problem problem_table[] = {
 
 	/* Bad or non-existent /lost+found.  Cannot reconnect */
 	{ PR_3_NO_LPF,
-	  N_("Bad or non-existent /@l.  Cannot reconnect\n"),
+	  N_("Bad or non-existent /@l.  Cannot reconnect.\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Could not expand /lost+found */
@@ -1050,7 +1138,7 @@ static const struct e2fsck_problem problem_table[] = {
 	  PROMPT_NONE, PR_PREEN_OK | PR_PREEN_NOMSG},
 
 	/* Block not used, but marked in bitmap */
-	{ PR_5_UNUSED_BLOCK,
+	{ PR_5_BLOCK_UNUSED,
 	  " -%b",
 	  PROMPT_NONE, PR_LATCH_BBITMAP | PR_PREEN_OK | PR_PREEN_NOMSG },
 		  
@@ -1070,7 +1158,7 @@ static const struct e2fsck_problem problem_table[] = {
 	  PROMPT_NONE, PR_PREEN_OK | PR_PREEN_NOMSG },
 
 	/* Inode not used, but marked in bitmap */
-	{ PR_5_UNUSED_INODE,
+	{ PR_5_INODE_UNUSED,
 	  " -%i",
 	  PROMPT_NONE, PR_LATCH_IBITMAP | PR_PREEN_OK | PR_PREEN_NOMSG },
 		  
@@ -1130,6 +1218,26 @@ static const struct e2fsck_problem problem_table[] = {
 	  "Error copying in replacement @b @B: %m\n",
 	  PROMPT_NONE, PR_FATAL },
 		  
+	/* Block range not used, but marked in bitmap */
+	{ PR_5_BLOCK_RANGE_UNUSED,
+	  " -(%b--%c)",
+	  PROMPT_NONE, PR_LATCH_BBITMAP | PR_PREEN_OK | PR_PREEN_NOMSG },
+		  
+	/* Block range used, but not marked used in bitmap */
+	{ PR_5_BLOCK_RANGE_USED,
+	  " +(%b--%c)",
+	  PROMPT_NONE, PR_LATCH_BBITMAP | PR_PREEN_OK | PR_PREEN_NOMSG },
+
+	/* Inode range not used, but marked in bitmap */
+	{ PR_5_INODE_RANGE_UNUSED,
+	  " -(%i--%j)",
+	  PROMPT_NONE, PR_LATCH_IBITMAP | PR_PREEN_OK | PR_PREEN_NOMSG },
+		  
+	/* Inode range used, but not marked used in bitmap */
+	{ PR_5_INODE_RANGE_USED,
+	  " +(%i--%j)",
+	  PROMPT_NONE, PR_LATCH_IBITMAP | PR_PREEN_OK | PR_PREEN_NOMSG },
+
 	{ 0 }
 };
 
@@ -1229,7 +1337,7 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 
 	ptr = find_problem(code);
 	if (!ptr) {
-		printf(_("Unhandled error code (%d)!\n"), code);
+		printf(_("Unhandled error code (0x%x)!\n"), code);
 		return 0;
 	}
 	def_yn = 1;
@@ -1264,7 +1372,8 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 	if (!suppress) {
 		message = ptr->e2p_description;
 		if (ctx->options & E2F_OPT_PREEN) {
-			printf("%s: ", ctx->device_name);
+			printf("%s: ", ctx->device_name ?
+			       ctx->device_name : ctx->filesystem_name);
 #if 0
 			if (ptr->e2p_preen_msg)
 				message = ptr->e2p_preen_msg;
