@@ -98,14 +98,14 @@ struct problem_context {
 /* Journal inode is invalid */
 #define PR_0_JOURNAL_BAD_INODE	0x00000F
 
-/* Superblock has a journal device (which we can't handle yet) */
-#define PR_0_JOURNAL_UNSUPP_DEV	0x000010
+/* The external journal has multiple filesystems (which we can't handle yet) */
+#define PR_0_JOURNAL_UNSUPP_MULTIFS 0x000010
 
-/* Superblock has a bad journal device */
-#define PR_0_JOURNAL_BAD_DEV	0x000011
+/* Can't find external journal */
+#define PR_0_CANT_FIND_JOURNAL	0x000011
 
-/* Superblock has a journal UUID (which we can't handle yet) */
-#define PR_0_JOURNAL_UNSUPP_UUID 0x000012
+/* External journal has bad superblock */
+#define PR_0_EXT_JOURNAL_BAD_SUPER 0x000012
 
 /* Superblock has a bad journal UUID */
 #define PR_0_JOURNAL_BAD_UUID	0x000013
@@ -122,11 +122,11 @@ struct problem_context {
 /* Superblock has recovery flag set but no journal */
 #define PR_0_JOURNAL_RECOVER_SET 0x000017
 
-/* Warning message about leaving data in the journal */
-#define PR_0_JOURNAL_RESET_JOURNAL 0x000018
+/* Journal has data, but recovery flag is clear */
+#define PR_0_JOURNAL_RECOVERY_CLEAR 0x000018
 
-/* Superblock recovery flag clear - journal needs to be reset */
-#define PR_0_JOURNAL_RESET_PROMPT 0x000019
+/* Ask if we should clear the journal */
+#define PR_0_JOURNAL_RESET_JOURNAL 0x000019
 
 /* Filesystem revision is 0, but feature flags are set */
 #define PR_0_FS_REV_LEVEL	0x00001A
@@ -155,6 +155,21 @@ struct problem_context {
 /* Journal has unsupported version number */
 #define PR_0_JOURNAL_UNSUPP_VERSION		0x000027
 
+/* Moving journal to hidden file */
+#define	PR_0_MOVE_JOURNAL			0x000028
+
+/* Error moving journal */
+#define	PR_0_ERR_MOVE_JOURNAL			0x000029
+
+/* Clearing V2 journal superblock */
+#define PR_0_CLEAR_V2_JOURNAL			0x00002A
+
+/* Run journal anyway */
+#define PR_0_JOURNAL_RUN			0x00002B
+
+/* Run journal anyway by default */
+#define PR_0_JOURNAL_RUN_DEFAULT		0x00002C
+
 /*
  * Pass 1 errors
  */
@@ -297,9 +312,6 @@ struct problem_context {
 /* Suppress messages prompt */
 #define PR_1_SUPPRESS_MESSAGES		0x01002D
 
-/* Filesystem contains large files, but has no such flag in sb */
-#define PR_1_FEATURE_LARGE_FILES 	0x01002E
-
 /* Imagic flag set on an inode when filesystem doesn't support it */
 #define PR_1_SET_IMAGIC			0x01002F
 
@@ -326,6 +338,39 @@ struct problem_context {
 
 /* Latch question which asks how to deal with low dtime inodes */
 #define PR_1_ORPHAN_LIST_REFUGEES	0x010037
+
+/* Error allocating refcount structure */
+#define PR_1_ALLOCATE_REFCOUNT		0x010038
+	
+/* Error reading Extended Attribute block */
+#define PR_1_READ_EA_BLOCK		0x010039
+
+/* Invalid Extended Attribute block */
+#define PR_1_BAD_EA_BLOCK		0x01003A
+
+/* Error reading Extended Attribute block while fixing refcount -- abort */
+#define PR_1_EXTATTR_READ_ABORT		0x01003B
+
+/* Extended attribute reference count incorrect */
+#define PR_1_EXTATTR_REFCOUNT		0x01003C
+
+/* Error writing Extended Attribute block while fixing refcount */ 
+#define PR_1_EXTATTR_WRITE		0x01003D
+
+/* Multiple EA blocks not supported */
+#define PR_1_EA_MULTI_BLOCK		0x01003E
+
+/* Error allocating EA region allocation structure */
+#define PR_1_EA_ALLOC_REGION		0x01003F
+	
+/* Error EA allocation collision */
+#define PR_1_EA_ALLOC_COLLISION		0x010040
+	
+/* Bad extended attribute name */
+#define PR_1_EA_BAD_NAME		0x010041
+
+/* Bad extended attribute value */
+#define PR_1_EA_BAD_VALUE		0x0100423
 	
 /*
  * Pass 1b errors
@@ -520,6 +565,12 @@ struct problem_context {
 /* Invalid fast symlink size */
 #define PR_2_SYMLINK_SIZE	0x020031
 
+/* i_file_acl (extended attribute) is bad */
+#define PR_2_FILE_ACL_BAD	0x020032
+
+/* Filesystem contains large files, but has no such flag in sb */
+#define PR_2_FEATURE_LARGE_FILES 0x020033
+
 /*
  * Pass 3 errors
  */
@@ -632,7 +683,7 @@ struct problem_context {
 #define PR_5_BLOCK_BITMAP_HEADER 	0x050003
 
 /* Block not used, but marked in bitmap */
-#define PR_5_UNUSED_BLOCK		0x050004
+#define PR_5_BLOCK_UNUSED		0x050004
 	  
 /* Block used, but not marked used in bitmap */
 #define PR_5_BLOCK_USED			0x050005
@@ -644,7 +695,7 @@ struct problem_context {
 #define PR_5_INODE_BITMAP_HEADER	0x050007
 
 /* Inode not used, but marked in bitmap */
-#define PR_5_UNUSED_INODE		0x050008
+#define PR_5_INODE_UNUSED		0x050008
 	  
 /* Inode used, but not marked used in bitmap */
 #define PR_5_INODE_USED			0x050009
@@ -679,6 +730,18 @@ struct problem_context {
 /* Error copying in replacement block bitmap */
 #define PR_5_COPY_BBITMAP_ERROR		0x050013
 
+/* Block range not used, but marked in bitmap */
+#define PR_5_BLOCK_RANGE_UNUSED		0x050014
+	  
+/* Block range used, but not marked used in bitmap */
+#define PR_5_BLOCK_RANGE_USED		0x050015
+
+/* Inode range not used, but marked in bitmap */
+#define PR_5_INODE_RANGE_UNUSED		0x050016
+	  
+/* Inode rangeused, but not marked used in bitmap */
+#define PR_5_INODE_RANGE_USED		0x050017
+
 /*
  * Function declarations
  */
