@@ -42,12 +42,10 @@ extern int optind;
 #include "../version.h"
 
 /* Command line options */
-static int blocksize = 0;
 static int swapfs = 0;
 static int normalize_swapfs = 0;
 static int cflag = 0;		/* check disk */
 static int show_version_only = 0;
-static int force = 0;
 static int verbose = 0;
 
 static int replace_bad_blocks = 0;
@@ -65,7 +63,7 @@ static void usage(e2fsck_t ctx)
 	fprintf(stderr,
 		_("Usage: %s [-panyrcdfvstFSV] [-b superblock] [-B blocksize]\n"
 		"\t\t[-I inode_buffer_blocks] [-P process_inode_size]\n"
-		"\t\t[-l|-L bad_blocks_file] [-C fd] device\n"),
+		"\t\t[-l|-L bad_blocks_file] [-C fd] [-j ext-journal] device\n"),
 		ctx->program_name);
 
 	fprintf(stderr, _("\nEmergency help:\n"
@@ -78,6 +76,7 @@ static void usage(e2fsck_t ctx)
 		" -v                   Be verbose\n"
 		" -b superblock        Use alternative superblock\n"
 		" -B blocksize         Force blocksize when looking for superblock\n"
+		" -j external-journal  Set location of the external journal\n"
 		" -l bad_blocks_file   Add to badblocks list\n"
 		" -L bad_blocks_file   Set badblocks list\n"
 		));
@@ -130,7 +129,9 @@ static void show_stats(e2fsck_t	ctx)
 		ctx->fs_ind_count, ctx->fs_dind_count, ctx->fs_tind_count);
 	printf (_("%8d blocks used (%d%%)\n"
 		"%8d bad blocks\n"), blocks_used,
-		100 * blocks_used / blocks, ctx->fs_badblocks_count);
+		(int) ((long long) 100 * blocks_used / blocks),
+		ctx->fs_badblocks_count);
+	printf(_("%8d large files\n"), ctx->large_files);
 	printf (_("\n%8d regular files\n"
 		"%8d directories\n"
 		"%8d character device files\n"
@@ -164,6 +165,8 @@ static void show_stats(e2fsck_t	ctx)
 		(blocks_used != 1) ? "s" : "",
 		100 * blocks_used / blocks, ctx->fs_badblocks_count,
 		ctx->fs_badblocks_count != 1 ? "s" : "");
+	printf(_("%8d large file%s\n"), ctx->large_files,
+	       (ctx->large_files != 1) ? "s" : "");
 	printf ("\n%8d regular file%s\n"
 		"%8d director%s\n"
 		"%8d character device file%s\n"
@@ -199,7 +202,7 @@ static void show_stats(e2fsck_t	ctx)
 static void check_mount(e2fsck_t ctx)
 {
 	errcode_t	retval;
-	int		mount_flags, cont, fd;
+	int		mount_flags, cont;
 
 	retval = ext2fs_check_if_mounted(ctx->filesystem_name, &mount_flags);
 	if (retval) {
@@ -208,23 +211,16 @@ static void check_mount(e2fsck_t ctx)
 			ctx->filesystem_name);
 		return;
 	}
-	if (!(mount_flags & EXT2_MF_MOUNTED))
+
+	/*
+	 * If the filesystem isn't mounted, or it's the root filesystem
+	 * and it's mounted read-only, then everything's fine.
+	 */
+	if ((!(mount_flags & EXT2_MF_MOUNTED)) ||
+	    ((mount_flags & EXT2_MF_ISROOT) &&
+	     (mount_flags & EXT2_MF_READONLY)))
 		return;
 
-#if (defined(__linux__) && defined(HAVE_MNTENT_H))
-	/*
-	 * If the root is mounted read-only, then /etc/mtab is
-	 * probably not correct; so we won't issue a warning based on
-	 * it.
-	 */
-	fd = open(MOUNTED, O_RDWR);
-	if (fd < 0) {
-		if (errno == EROFS)
-			return;
-	} else
-		close(fd);
-#endif
-	
 	if (ctx->options & E2F_OPT_READONLY) {
 		printf(_("Warning!  %s is mounted.\n"), ctx->filesystem_name);
 		return;
@@ -255,7 +251,8 @@ static void check_if_skip(e2fsck_t ctx)
 	const char *reason = NULL;
 	unsigned int reason_arg = 0;
 	
-	if (force || bad_blocks_file || cflag || swapfs)
+	if ((ctx->options & E2F_OPT_FORCE) || bad_blocks_file ||
+	    cflag || swapfs)
 		return;
 	
 	if (fs->super->s_state & EXT2_ERROR_FS)
@@ -285,6 +282,8 @@ static void check_if_skip(e2fsck_t ctx)
 	       fs->super->s_blocks_count - fs->super->s_free_blocks_count,
 	       fs->super->s_blocks_count);
 	ext2fs_close(fs);
+	ctx->fs = NULL;
+	e2fsck_free_context(ctx);
 	exit(FSCK_OK);
 }
 
@@ -448,7 +447,6 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 #ifdef MTRACE
 	extern void	*mallwatch;
 #endif
-	char		*oldpath = getenv("PATH");
 	e2fsck_t	ctx;
 	errcode_t	retval;
 #ifdef HAVE_SIGNAL_H
@@ -461,21 +459,6 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 
 	*ret_ctx = ctx;
 
-	/* Update our PATH to include /sbin  */
-	if (oldpath) {
-		char *newpath;
-
-		newpath = (char *) malloc(sizeof (PATH_SET) + 1 +
-					  strlen (oldpath));
-		if (!newpath)
-			fatal_error(ctx, "Couldn't malloc() newpath");
-		strcpy (newpath, PATH_SET);
-		strcat (newpath, ":");
-		strcat (newpath, oldpath);
-		putenv (newpath);
-	} else
-		putenv (PATH_SET);
-
 	setbuf(stdout, NULL);
 	setbuf(stderr, NULL);
 	initialize_ext2_error_table();
@@ -484,7 +467,7 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 		ctx->program_name = *argv;
 	else
 		ctx->program_name = "e2fsck";
-	while ((c = getopt (argc, argv, "panyrcC:B:dfvtFVM:b:I:P:l:L:N:Ss")) != EOF)
+	while ((c = getopt (argc, argv, "panyrcC:B:dfvtFVM:b:I:j:P:l:L:N:Ss")) != EOF)
 		switch (c) {
 		case 'C':
 			ctx->progress = e2fsck_update_progress;
@@ -528,7 +511,8 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 #endif
 			break;
 		case 'c':
-			cflag++;
+			if (cflag++)
+				ctx->options |= E2F_OPT_WRITECHECK;
 			ctx->options |= E2F_OPT_CHECKBLOCKS;
 			break;
 		case 'r':
@@ -536,13 +520,17 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 			break;
 		case 'b':
 			ctx->use_superblock = atoi(optarg);
+			ctx->flags |= E2F_FLAG_SB_SPECIFIED;
 			break;
 		case 'B':
-			blocksize = atoi(optarg);
+			ctx->blocksize = atoi(optarg);
 			break;
 		case 'I':
 			ctx->inode_buffer_blocks = atoi(optarg);
 			break;
+		case 'j':
+			ctx->journal_name = optarg;
+			break;
 		case 'P':
 			ctx->process_inode_size = atoi(optarg);
 			break;
@@ -559,7 +547,7 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 			ctx->options |= E2F_OPT_DEBUG;
 			break;
 		case 'f':
-			force = 1;
+			ctx->options |= E2F_OPT_FORCE;
 			break;
 		case 'F':
 			flush = 1;
@@ -604,8 +592,7 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 		ctx->options |= E2F_OPT_READONLY;
 	ctx->filesystem_name = argv[optind];
 	if (flush) {
-		int	fd = open(ctx->filesystem_name, O_RDONLY, 0);
-
+		fd = open(ctx->filesystem_name, O_RDONLY, 0);
 		if (fd < 0) {
 			com_err("open", errno,
 				_("while opening %s for flushing"),
@@ -624,11 +611,16 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 	if (swapfs) {
 		if (cflag || bad_blocks_file) {
 			fprintf(stderr, _("Incompatible options not "
-				"allowed when byte-swapping.\n"));
+					  "allowed when byte-swapping.\n"));
 			exit(FSCK_USAGE);
 		}
 	}
 #endif
+	if (cflag && bad_blocks_file) {
+		fprintf(stderr, _("The -c and the -l/-L options may "
+				  "not be both used at the same time.\n"));
+		exit(FSCK_USAGE);
+	}
 #ifdef HAVE_SIGNAL_H
 	/*
 	 * Set up signal action
@@ -643,6 +635,24 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 	sa.sa_handler = signal_progress_off;
 	sigaction(SIGUSR2, &sa, 0);
 #endif
+
+	/* Update our PATH to include /sbin if we need to run badblocks  */
+	if (cflag) {
+		char *oldpath = getenv("PATH");
+		if (oldpath) {
+			char *newpath;
+
+			newpath = (char *) malloc(sizeof (PATH_SET) + 1 +
+						  strlen (oldpath));
+			if (!newpath)
+				fatal_error(ctx, "Couldn't malloc() newpath");
+			strcpy (newpath, PATH_SET);
+			strcat (newpath, ":");
+			strcat (newpath, oldpath);
+			putenv (newpath);
+		} else
+			putenv (PATH_SET);
+	}
 	return 0;
 }
 
@@ -672,6 +682,7 @@ int main (int argc, char *argv[])
 #endif
 #ifdef ENABLE_NLS
 	setlocale(LC_MESSAGES, "");
+	setlocale(LC_CTYPE, "");
 	bindtextdomain(NLS_CAT_NAME, LOCALEDIR);
 	textdomain(NLS_CAT_NAME);
 #endif
@@ -696,9 +707,8 @@ int main (int argc, char *argv[])
 #endif
 
 	if (!(ctx->options & E2F_OPT_PREEN) || show_version_only)
-		fprintf (stderr, _("e2fsck %s, %s for EXT2 FS %s, %s\n"),
-			 my_ver_string, my_ver_date, EXT2FS_VERSION,
-			 EXT2FS_DATE);
+		fprintf (stderr, "e2fsck %s (%s)\n", my_ver_string,
+			 my_ver_date);
 
 	if (show_version_only) {
 		fprintf(stderr, _("\tUsing %s, %s\n"),
@@ -727,9 +737,10 @@ restart:
 	if ((ctx->options & E2F_OPT_READONLY) == 0)
 		flags |= EXT2_FLAG_RW;
 
-	if (ctx->superblock && blocksize) {
+	if (ctx->superblock && ctx->blocksize) {
 		retval = ext2fs_open(ctx->filesystem_name, flags,
-				     ctx->superblock, blocksize, io_ptr, &fs);
+				     ctx->superblock, ctx->blocksize,
+				     io_ptr, &fs);
 	} else if (ctx->superblock) {
 		for (i=0; possible_block_sizes[i]; i++) {
 			retval = ext2fs_open(ctx->filesystem_name, flags,
@@ -742,14 +753,15 @@ restart:
 	} else 
 		retval = ext2fs_open(ctx->filesystem_name, flags, 
 				     0, 0, io_ptr, &fs);
-	if (!ctx->superblock && !(ctx->options & E2F_OPT_PREEN) && 
+	if (!ctx->superblock && !(ctx->options & E2F_OPT_PREEN) &&
+	    !(ctx->flags & E2F_FLAG_SB_SPECIFIED) &&
 	    ((retval == EXT2_ET_BAD_MAGIC) ||
 	     ((retval == 0) && ext2fs_check_desc(fs)))) {
 		if (!fs || (fs->group_desc_count > 1)) {
 			printf(_("%s trying backup blocks...\n"),
 			       retval ? _("Couldn't find ext2 superblock,") :
 			       _("Group descriptors look bad..."));
-			ctx->superblock = get_backup_sb(fs);
+			get_backup_sb(ctx, fs, ctx->filesystem_name, io_ptr);
 			if (fs)
 				ext2fs_close(fs);
 			goto restart;
@@ -819,7 +831,6 @@ restart:
 		com_err(ctx->program_name, retval,
 			_("while checking ext3 journal for %s"),
 			ctx->device_name);
-		ext2fs_close(ctx->fs);
 		fatal_error(ctx, 0);
 	}
 
@@ -961,8 +972,11 @@ restart:
 	}
 	if (ext2fs_test_valid(fs))
 		fs->flags &= ~EXT2_FLAG_MASTER_SB_ONLY;
-	else
+	else {
+		printf(_("\n%s: ********** WARNING: Filesystem still has "
+			 "errors **********\n\n"), ctx->device_name);
 		exit_value = FSCK_UNCORRECTED;
+	}
 	if (!(ctx->options & E2F_OPT_READONLY)) {
 		if (ext2fs_test_valid(fs)) {
 			if (!(sb->s_state & EXT2_VALID_FS))
@@ -978,14 +992,14 @@ restart:
 
 	e2fsck_write_bitmaps(ctx);
 	
+	ext2fs_close(fs);
+	ctx->fs = NULL;
+	e2fsck_free_context(ctx);
+	
 #ifdef RESOURCE_TRACK
 	if (ctx->options & E2F_OPT_TIME)
 		print_resource_track(NULL, &ctx->global_rtrack);
 #endif
 
-	ext2fs_close(fs);
-	ctx->fs = NULL;
-	e2fsck_free_context(ctx);
-	
 	return exit_value;
 }
