@@ -14,6 +14,9 @@
  * 93/10/30	- Creation
  */
 
+#define _LARGEFILE_SOURCE
+#define _LARGEFILE64_SOURCE
+
 #if HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -29,6 +32,12 @@
 
 #include "e2p.h"
 
+#ifdef O_LARGEFILE
+#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK|O_LARGEFILE)
+#else
+#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK)
+#endif
+
 int fsetflags (const char * name, unsigned long flags)
 {
 #if HAVE_CHFLAGS
@@ -52,7 +61,7 @@ int fsetflags (const char * name, unsigned long flags)
 #if HAVE_EXT2_IOCTLS
 	int fd, r, f;
 
-	fd = open (name, O_RDONLY|O_NONBLOCK);
+	fd = open (name, OPEN_FLAGS);
 	if (fd == -1)
 		return -1;
 	f = (int) flags;
