@@ -14,6 +14,9 @@
  * 93/10/30	- Creation
  */
 
+#define _LARGEFILE_SOURCE
+#define _LARGEFILE64_SOURCE
+
 #if HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -25,12 +28,18 @@
 
 #include "e2p.h"
 
+#ifdef O_LARGEFILE
+#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK|O_LARGEFILE)
+#else
+#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK)
+#endif
+
 int fsetversion (const char * name, unsigned long version)
 {
 #if HAVE_EXT2_IOCTLS
 	int fd, r, ver;
 
-	fd = open (name, O_RDONLY|O_NONBLOCK);
+	fd = open (name, OPEN_FLAGS);
 	if (fd == -1)
 		return -1;
 	ver = (int) version;
