@@ -15,12 +15,7 @@
 #include <unistd.h>
 #endif
 
-#if EXT2_FLAT_INCLUDES
 #include "ext2_fs.h"
-#else
-#include <linux/ext2_fs.h>
-#endif
-
 #include "ext2fs.h"
 
 struct block_context {
@@ -64,18 +59,24 @@ static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	ctx->errcode = io_channel_read_blk(ctx->fs->io, *ind_block,
-					   1, ctx->ind_buf);
+	if (ctx->fs->flags & EXT2_FLAG_IMAGE_FILE) {
+		ctx->errcode = 0;
+		memset(ctx->ind_buf, 0, ctx->fs->blocksize);
+	} else
+		ctx->errcode = io_channel_read_blk(ctx->fs->io, *ind_block,
+						   1, ctx->ind_buf);
 	if (ctx->errcode) {
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
-	    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_READ)) {
+#ifdef EXT2FS_ENABLE_SWAPFS
+	if (ctx->fs->flags & (EXT2_FLAG_SWAP_BYTES |
+			      EXT2_FLAG_SWAP_BYTES_READ)) {
 		block_nr = (blk_t *) ctx->ind_buf;
 		for (i = 0; i < limit; i++, block_nr++)
 			*block_nr = ext2fs_swab32(*block_nr);
 	}
+#endif
 	block_nr = (blk_t *) ctx->ind_buf;
 	offset = 0;
 	if (ctx->flags & BLOCK_FLAG_APPEND) {
@@ -105,13 +106,16 @@ static int block_iterate_ind(blk_t *ind_block, blk_t ref_block,
 			offset += sizeof(blk_t);
 		}
 	}
-	if (changed & BLOCK_CHANGED) {
-		if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
-		    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_WRITE)) {
+	if (!(ctx->fs->flags & EXT2_FLAG_IMAGE_FILE) &&
+	    (changed & BLOCK_CHANGED)) {
+#ifdef EXT2FS_ENABLE_SWAPFS
+		if (ctx->fs->flags & (EXT2_FLAG_SWAP_BYTES |
+				      EXT2_FLAG_SWAP_BYTES_WRITE)) {
 			block_nr = (blk_t *) ctx->ind_buf;
 			for (i = 0; i < limit; i++, block_nr++)
 				*block_nr = ext2fs_swab32(*block_nr);
 		}
+#endif
 		ctx->errcode = io_channel_write_blk(ctx->fs->io, *ind_block,
 						    1, ctx->ind_buf);
 		if (ctx->errcode)
@@ -134,8 +138,8 @@ static int block_iterate_dind(blk_t *dind_block, blk_t ref_block,
 	blk_t	*block_nr;
 
 	limit = ctx->fs->blocksize >> 2;
-	if (!(ctx->flags & BLOCK_FLAG_DEPTH_TRAVERSE) &&
-	    !(ctx->flags & BLOCK_FLAG_DATA_ONLY))
+	if (!(ctx->flags & (BLOCK_FLAG_DEPTH_TRAVERSE |
+			    BLOCK_FLAG_DATA_ONLY)))
 		ret = (*ctx->func)(ctx->fs, dind_block,
 				   BLOCK_COUNT_DIND, ref_block,
 				   ref_offset, ctx->priv_data);
@@ -149,18 +153,24 @@ static int block_iterate_dind(blk_t *dind_block, blk_t ref_block,
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	ctx->errcode = io_channel_read_blk(ctx->fs->io, *dind_block,
-					   1, ctx->dind_buf);
+	if (ctx->fs->flags & EXT2_FLAG_IMAGE_FILE) {
+		ctx->errcode = 0;
+		memset(ctx->dind_buf, 0, ctx->fs->blocksize);
+	} else
+		ctx->errcode = io_channel_read_blk(ctx->fs->io, *dind_block,
+						   1, ctx->dind_buf);
 	if (ctx->errcode) {
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
-	    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_READ)) {
+#ifdef EXT2FS_ENABLE_SWAPFS
+	if (ctx->fs->flags & (EXT2_FLAG_SWAP_BYTES |
+			      EXT2_FLAG_SWAP_BYTES_READ)) {
 		block_nr = (blk_t *) ctx->dind_buf;
 		for (i = 0; i < limit; i++, block_nr++)
 			*block_nr = ext2fs_swab32(*block_nr);
 	}
+#endif
 	block_nr = (blk_t *) ctx->dind_buf;
 	offset = 0;
 	if (ctx->flags & BLOCK_FLAG_APPEND) {
@@ -192,13 +202,16 @@ static int block_iterate_dind(blk_t *dind_block, blk_t ref_block,
 			offset += sizeof(blk_t);
 		}
 	}
-	if (changed & BLOCK_CHANGED) {
-		if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
-		    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_WRITE)) {
+	if (!(ctx->fs->flags & EXT2_FLAG_IMAGE_FILE) &&
+	    (changed & BLOCK_CHANGED)) {
+#ifdef EXT2FS_ENABLE_SWAPFS
+		if (ctx->fs->flags & (EXT2_FLAG_SWAP_BYTES |
+				      EXT2_FLAG_SWAP_BYTES_WRITE)) {
 			block_nr = (blk_t *) ctx->dind_buf;
 			for (i = 0; i < limit; i++, block_nr++)
 				*block_nr = ext2fs_swab32(*block_nr);
 		}
+#endif
 		ctx->errcode = io_channel_write_blk(ctx->fs->io, *dind_block,
 						    1, ctx->dind_buf);
 		if (ctx->errcode)
@@ -221,8 +234,8 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 	blk_t	*block_nr;
 
 	limit = ctx->fs->blocksize >> 2;
-	if (!(ctx->flags & BLOCK_FLAG_DEPTH_TRAVERSE) &&
-	    !(ctx->flags & BLOCK_FLAG_DATA_ONLY))
+	if (!(ctx->flags & (BLOCK_FLAG_DEPTH_TRAVERSE |
+			    BLOCK_FLAG_DATA_ONLY)))
 		ret = (*ctx->func)(ctx->fs, tind_block,
 				   BLOCK_COUNT_TIND, ref_block,
 				   ref_offset, ctx->priv_data);
@@ -236,18 +249,24 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	ctx->errcode = io_channel_read_blk(ctx->fs->io, *tind_block,
-					   1, ctx->tind_buf);
+	if (ctx->fs->flags & EXT2_FLAG_IMAGE_FILE) {
+		ctx->errcode = 0;
+		memset(ctx->tind_buf, 0, ctx->fs->blocksize);
+	} else
+		ctx->errcode = io_channel_read_blk(ctx->fs->io, *tind_block,
+						   1, ctx->tind_buf);
 	if (ctx->errcode) {
 		ret |= BLOCK_ERROR;
 		return ret;
 	}
-	if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
-	    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_READ)) {
+#ifdef EXT2FS_ENABLE_SWAPFS
+	if (ctx->fs->flags & (EXT2_FLAG_SWAP_BYTES |
+			      EXT2_FLAG_SWAP_BYTES_READ)) {
 		block_nr = (blk_t *) ctx->tind_buf;
 		for (i = 0; i < limit; i++, block_nr++)
 			*block_nr = ext2fs_swab32(*block_nr);
 	}
+#endif
 	block_nr = (blk_t *) ctx->tind_buf;
 	offset = 0;
 	if (ctx->flags & BLOCK_FLAG_APPEND) {
@@ -279,13 +298,16 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 			offset += sizeof(blk_t);
 		}
 	}
-	if (changed & BLOCK_CHANGED) {
-		if ((ctx->fs->flags & EXT2_FLAG_SWAP_BYTES) ||
-		    (ctx->fs->flags & EXT2_FLAG_SWAP_BYTES_WRITE)) {
+	if (!(ctx->fs->flags & EXT2_FLAG_IMAGE_FILE) &&
+	    (changed & BLOCK_CHANGED)) {
+#ifdef EXT2FS_ENABLE_SWAPFS
+		if (ctx->fs->flags & (EXT2_FLAG_SWAP_BYTES |
+				      EXT2_FLAG_SWAP_BYTES_WRITE)) {
 			block_nr = (blk_t *) ctx->tind_buf;
 			for (i = 0; i < limit; i++, block_nr++)
 				*block_nr = ext2fs_swab32(*block_nr);
 		}
+#endif
 		ctx->errcode = io_channel_write_blk(ctx->fs->io, *tind_block,
 						    1, ctx->tind_buf);
 		if (ctx->errcode)
@@ -302,7 +324,7 @@ static int block_iterate_tind(blk_t *tind_block, blk_t ref_block,
 }
 	
 errcode_t ext2fs_block_iterate2(ext2_filsys fs,
-				ino_t	ino,
+				ext2_ino_t ino,
 				int	flags,
 				char *block_buf,
 				int (*func)(ext2_filsys fs,
@@ -366,7 +388,7 @@ errcode_t ext2fs_block_iterate2(ext2_filsys fs,
 	    !(flags & BLOCK_FLAG_DATA_ONLY)) {
 		ctx.errcode = ext2fs_read_inode(fs, ino, &inode);
 		if (ctx.errcode)
-			goto abort;
+			goto abort_exit;
 		got_inode = 1;
 		if (inode.osd1.hurd1.h_i_translator) {
 			ret |= (*ctx.func)(fs,
@@ -374,7 +396,7 @@ errcode_t ext2fs_block_iterate2(ext2_filsys fs,
 					   BLOCK_COUNT_TRANSLATOR,
 					   0, 0, priv_data);
 			if (ret & BLOCK_ABORT)
-				goto abort;
+				goto abort_exit;
 		}
 	}
 	
@@ -386,31 +408,31 @@ errcode_t ext2fs_block_iterate2(ext2_filsys fs,
 			ret |= (*ctx.func)(fs, &blocks[i],
 					    ctx.bcount, 0, i, priv_data);
 			if (ret & BLOCK_ABORT)
-				goto abort;
+				goto abort_exit;
 		}
 	}
 	if (*(blocks + EXT2_IND_BLOCK) || (flags & BLOCK_FLAG_APPEND)) {
 		ret |= block_iterate_ind(blocks + EXT2_IND_BLOCK,
 					 0, EXT2_IND_BLOCK, &ctx);
 		if (ret & BLOCK_ABORT)
-			goto abort;
+			goto abort_exit;
 	} else
 		ctx.bcount += limit;
 	if (*(blocks + EXT2_DIND_BLOCK) || (flags & BLOCK_FLAG_APPEND)) {
 		ret |= block_iterate_dind(blocks + EXT2_DIND_BLOCK,
 					  0, EXT2_DIND_BLOCK, &ctx);
 		if (ret & BLOCK_ABORT)
-			goto abort;
+			goto abort_exit;
 	} else
 		ctx.bcount += limit * limit;
 	if (*(blocks + EXT2_TIND_BLOCK) || (flags & BLOCK_FLAG_APPEND)) {
 		ret |= block_iterate_tind(blocks + EXT2_TIND_BLOCK,
 					  0, EXT2_TIND_BLOCK, &ctx);
 		if (ret & BLOCK_ABORT)
-			goto abort;
+			goto abort_exit;
 	}
 
-abort:
+abort_exit:
 	if (ret & BLOCK_CHANGED) {
 		if (!got_inode) {
 			retval = ext2fs_read_inode(fs, ino, &inode);
@@ -443,7 +465,7 @@ struct xlate {
 };
 
 #ifdef __TURBOC__
-#pragma argsused
+ #pragma argsused
 #endif
 static int xlate_func(ext2_filsys fs, blk_t *blocknr, e2_blkcnt_t blockcnt,
 		      blk_t ref_block, int ref_offset, void *priv_data)
@@ -454,7 +476,7 @@ static int xlate_func(ext2_filsys fs, blk_t *blocknr, e2_blkcnt_t blockcnt,
 }
 
 errcode_t ext2fs_block_iterate(ext2_filsys fs,
-			       ino_t	ino,
+			       ext2_ino_t ino,
 			       int	flags,
 			       char *block_buf,
 			       int (*func)(ext2_filsys fs,
