@@ -1,5 +1,5 @@
 /*
- * bmap.c --- logical to phiscal block mapping
+ * bmap.c --- logical to physical block mapping
  *
  * Copyright (C) 1997 Theodore Ts'o.
  *
@@ -15,12 +15,7 @@
 #include <unistd.h>
 #endif
 
-#if EXT2_FLAT_INCLUDES
 #include "ext2_fs.h"
-#else
-#include <linux/ext2_fs.h>
-#endif
-
 #include "ext2fs.h"
 
 #if defined(__GNUC__) && !defined(NO_INLINE_FUNCS)
@@ -29,7 +24,7 @@
 #define _BMAP_INLINE_
 #endif
 
-extern errcode_t ext2fs_bmap(ext2_filsys fs, ino_t ino,
+extern errcode_t ext2fs_bmap(ext2_filsys fs, ext2_ino_t ino,
 			     struct ext2_inode *inode, 
 			     char *block_buf, int bmap_flags,
 			     blk_t block, blk_t *phys_blk);
@@ -38,7 +33,7 @@ extern errcode_t ext2fs_bmap(ext2_filsys fs, ino_t ino,
 
 #define inode_bmap(inode, nr) ((inode)->i_block[(nr)])
 
-static errcode_t _BMAP_INLINE_ block_ind_bmap(ext2_filsys fs, int flags, 
+static _BMAP_INLINE_ errcode_t block_ind_bmap(ext2_filsys fs, int flags, 
 					      blk_t ind, char *block_buf, 
 					      int *blocks_alloc,
 					      blk_t nr, blk_t *ret_blk)
@@ -56,9 +51,11 @@ static errcode_t _BMAP_INLINE_ block_ind_bmap(ext2_filsys fs, int flags,
 
 	b = ((blk_t *) block_buf)[nr];
 
+#ifdef EXT2FS_ENABLE_SWAPFS
 	if ((fs->flags & EXT2_FLAG_SWAP_BYTES) ||
 	    (fs->flags & EXT2_FLAG_SWAP_BYTES_READ))
 		b = ext2fs_swab32(b);
+#endif
 
 	if (!b && (flags & BMAP_ALLOC)) {
 		b = nr ? ((blk_t *) block_buf)[nr-1] : 0;
@@ -67,10 +64,12 @@ static errcode_t _BMAP_INLINE_ block_ind_bmap(ext2_filsys fs, int flags,
 		if (retval)
 			return retval;
 
+#ifdef EXT2FS_ENABLE_SWAPFS
 		if ((fs->flags & EXT2_FLAG_SWAP_BYTES) ||
 		    (fs->flags & EXT2_FLAG_SWAP_BYTES_WRITE))
 			((blk_t *) block_buf)[nr] = ext2fs_swab32(b);
 		else
+#endif
 			((blk_t *) block_buf)[nr] = b;
 
 		retval = io_channel_write_blk(fs->io, ind, 1, block_buf);
@@ -84,7 +83,7 @@ static errcode_t _BMAP_INLINE_ block_ind_bmap(ext2_filsys fs, int flags,
 	return 0;
 }
 
-static errcode_t _BMAP_INLINE_ block_dind_bmap(ext2_filsys fs, int flags,
+static _BMAP_INLINE_ errcode_t block_dind_bmap(ext2_filsys fs, int flags,
 					       blk_t dind, char *block_buf, 
 					       int *blocks_alloc,
 					       blk_t nr, blk_t *ret_blk)
@@ -104,7 +103,7 @@ static errcode_t _BMAP_INLINE_ block_dind_bmap(ext2_filsys fs, int flags,
 	return retval;
 }
 
-static errcode_t _BMAP_INLINE_ block_tind_bmap(ext2_filsys fs, int flags,
+static _BMAP_INLINE_ errcode_t block_tind_bmap(ext2_filsys fs, int flags,
 					       blk_t tind, char *block_buf, 
 					       int *blocks_alloc,
 					       blk_t nr, blk_t *ret_blk)
@@ -124,7 +123,7 @@ static errcode_t _BMAP_INLINE_ block_tind_bmap(ext2_filsys fs, int flags,
 	return retval;
 }
 
-errcode_t ext2fs_bmap(ext2_filsys fs, ino_t ino, struct ext2_inode *inode,
+errcode_t ext2fs_bmap(ext2_filsys fs, ext2_ino_t ino, struct ext2_inode *inode,
 		      char *block_buf, int bmap_flags, blk_t block,
 		      blk_t *phys_blk)
 {
