@@ -14,12 +14,7 @@
 #include <unistd.h>
 #endif
 
-#if EXT2_FLAT_INCLUDES
 #include "ext2_fs.h"
-#else
-#include <linux/ext2_fs.h>
-#endif
-
 #include "ext2fsP.h"
 
 static void ext2fs_free_inode_cache(struct ext2_inode_cache *icache);
@@ -35,6 +30,8 @@ void ext2fs_free(ext2_filsys fs)
 		ext2fs_free_mem((void **) &fs->device_name);
 	if (fs->super)
 		ext2fs_free_mem((void **) &fs->super);
+	if (fs->orig_super)
+		ext2fs_free_mem((void **) &fs->orig_super);
 	if (fs->group_desc)
 		ext2fs_free_mem((void **) &fs->group_desc);
 	if (fs->block_map)
