@@ -9,6 +9,9 @@
  * %End-Header%
  */
 
+#define _LARGEFILE_SOURCE
+#define _LARGEFILE64_SOURCE
+
 #include <stdio.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
@@ -17,9 +20,6 @@
 #include <errno.h>
 #endif
 #include <fcntl.h>
-#ifdef HAVE_LINUX_FS_H
-#include <linux/fs.h>
-#endif
 #ifdef HAVE_LINUX_FD_H
 #include <sys/ioctl.h>
 #include <linux/fd.h>
@@ -29,12 +29,11 @@
 #include <sys/disklabel.h>
 #endif /* HAVE_SYS_DISKLABEL_H */
 
-#if EXT2_FLAT_INCLUDES
-#include "ext2_fs.h"
-#else
-#include <linux/ext2_fs.h>
+#if defined(__linux__) && defined(_IO) && !defined(BLKGETSIZE)
+#define BLKGETSIZE _IO(0x12,96)	/* return device size */
 #endif
 
+#include "ext2_fs.h"
 #include "ext2fs.h"
 
 static int valid_offset (int fd, ext2_loff_t offset)
@@ -56,7 +55,7 @@ errcode_t ext2fs_get_device_size(const char *file, int blocksize,
 {
 	int	fd;
 #ifdef BLKGETSIZE
-	long	size;
+	unsigned long	size;
 #endif
 	ext2_loff_t high, low;
 #ifdef FDGETPRM
@@ -69,7 +68,11 @@ errcode_t ext2fs_get_device_size(const char *file, int blocksize,
 	char ch;
 #endif /* HAVE_SYS_DISKLABEL_H */
 
+#ifdef HAVE_OPEN64
+	fd = open64(file, O_RDONLY);
+#else
 	fd = open(file, O_RDONLY);
+#endif
 	if (fd < 0)
 		return errno;
 
@@ -130,3 +133,25 @@ errcode_t ext2fs_get_device_size(const char *file, int blocksize,
 	*retblocks = (low + 1) / blocksize;
 	return 0;
 }
+
+#ifdef DEBUG
+int main(int argc, char **argv)
+{
+	blk_t	blocks;
+	int	retval;
+	
+	if (argc < 2) {
+		fprintf(stderr, "Usage: %s device\n", argv[0]);
+		exit(1);
+	}
+
+	retval = ext2fs_get_device_size(argv[1], 1024, &blocks);
+	if (retval) {
+		com_err(argv[0], retval,
+			"while calling ext2fs_get_device_size");
+		exit(1);
+	}
+	printf("Device %s has %d 1k blocks.\n", argv[1], blocks);
+	exit(0);
+}
+#endif
