@@ -43,29 +43,42 @@
 static errcode_t check_mntent_file(const char *mtab_file, const char *file, 
 				   int *mount_flags, char *mtpt, int mtlen)
 {
-	FILE * f;
-	struct mntent * mnt;
-	int	fd;
+	struct mntent 	*mnt;
+	struct stat	st_buf;
+	errcode_t	retval = 0;
+	dev_t		file_dev;
+	FILE 		*f;
+	int		fd;
 
 	*mount_flags = 0;
 	if ((f = setmntent (mtab_file, "r")) == NULL)
 		return errno;
-	while ((mnt = getmntent (f)) != NULL)
+	file_dev = 0;
+#ifndef __GNU__ /* The GNU hurd is broken with respect to stat devices */
+	if (stat(file, &st_buf) == 0)
+		file_dev = st_buf.st_rdev;
+#endif	
+	while ((mnt = getmntent (f)) != NULL) {
 		if (strcmp(file, mnt->mnt_fsname) == 0)
 			break;
+#ifndef __GNU__
+		if (file_dev && (stat(mnt->mnt_fsname, &st_buf) == 0) &&
+		    file_dev == st_buf.st_rdev)
+			break;
+#endif
+	}
 
 	if (mnt == 0) {
 #ifndef __GNU__ /* The GNU hurd is broken with respect to stat devices */
-		struct stat st_root, st_file;
 		/*
 		 * Do an extra check to see if this is the root device.  We
-		 * can't trust /etc/fstab, and /proc/mounts will only list
+		 * can't trust /etc/mtab, and /proc/mounts will only list
 		 * /dev/root for the root filesystem.  Argh.  Instead we
 		 * check if the given device has the same major/minor number
 		 * as the device that the root directory is on.
 		 */
-		if (stat("/", &st_root) == 0 && stat(file, &st_file) == 0) {
-			if (st_root.st_dev == st_file.st_rdev) {
+		if (file_dev && stat("/", &st_buf) == 0) {
+			if (st_buf.st_dev == file_dev) {
 				*mount_flags = EXT2_MF_MOUNTED;
 				if (mtpt)
 					strncpy(mtpt, "/", mtlen);
@@ -73,9 +86,34 @@ static errcode_t check_mntent_file(const char *mtab_file, const char *file,
 			}
 		}
 #endif
-		endmntent (f);
-		return 0;
+		goto exit;
 	}
+#ifndef __GNU__ /* The GNU hurd is deficient; what else is new? */
+	/* Validate the entry in case /etc/mtab is out of date */
+	/* 
+	 * We need to be paranoid, because some broken distributions
+	 * (read: Slackware) don't initialize /etc/mtab before checking
+	 * all of the non-root filesystems on the disk.
+	 */
+	if (stat(mnt->mnt_dir, &st_buf) < 0) {
+		retval = errno;
+		if (retval == ENOENT) {
+#ifdef DEBUG
+			printf("Bogus entry in %s!  (%s does not exist)\n",
+			       mtab_file, mnt->mnt_dir);
+#endif
+			retval = 0;
+		}
+		goto exit;
+	}
+	if (file_dev && (st_buf.st_dev != file_dev)) {
+#ifdef DEBUG
+		printf("Bogus entry in %s!  (%s not mounted on %s)\n",
+		       mtab_file, file, mnt->mnt_dir);
+#endif
+		goto exit;
+	}
+#endif
 	*mount_flags = EXT2_MF_MOUNTED;
 	
 	/* Check to see if the ro option is set */
@@ -102,8 +140,54 @@ is_root:
 			close(fd);
 		(void) unlink(TEST_FILE);
 	}
+	retval = 0;
+exit:
 	endmntent (f);
-	return 0;
+	return retval;
+}
+
+/*
+ * Check to see if we're dealing with the swap device.
+ */
+static int is_swap_device(const char *file)
+{
+	FILE		*f;
+	char		buf[1024], *cp;
+	dev_t		file_dev;
+	struct stat	st_buf;
+	int		ret = 0;
+
+	file_dev = 0;
+#ifndef __GNU__ /* The GNU hurd is broken with respect to stat devices */
+	if (stat(file, &st_buf) == 0)
+		file_dev = st_buf.st_rdev;
+#endif	
+
+	if (!(f = fopen("/proc/swaps", "r")))
+		return 0;
+	/* Skip the first line */
+	fgets(buf, sizeof(buf), f);
+	while (!feof(f)) {
+		if (!fgets(buf, sizeof(buf), f))
+			break;
+		if ((cp = strchr(buf, ' ')) != NULL)
+			*cp = 0;
+		if ((cp = strchr(buf, '\t')) != NULL)
+			*cp = 0;
+		if (strcmp(buf, file) == 0) {
+			ret++;
+			break;
+		}
+#ifndef __GNU__
+		if (file_dev && (stat(buf, &st_buf) == 0) &&
+		    file_dev == st_buf.st_rdev) {
+			ret++;
+			break;
+		}
+#endif
+	}
+	fclose(f);
+	return ret;
 }
 
 static errcode_t check_mntent(const char *file, int *mount_flags,
@@ -111,6 +195,12 @@ static errcode_t check_mntent(const char *file, int *mount_flags,
 {
 	errcode_t	retval;
 
+#ifdef DEBUG
+	retval = check_mntent_file("/tmp/mtab", file, mount_flags,
+				   mtpt, mtlen);
+	if (retval == 0)
+		return 0;
+#endif
 #ifdef __linux__
 	retval = check_mntent_file("/proc/mounts", file, mount_flags,
 				   mtpt, mtlen);
@@ -171,6 +261,11 @@ static errcode_t check_getmntinfo(const char *file, int *mount_flags,
 errcode_t ext2fs_check_mount_point(const char *device, int *mount_flags,
 				  char *mtpt, int mtlen)
 {
+	if (is_swap_device(device)) {
+		*mount_flags = EXT2_MF_MOUNTED | EXT2_MF_SWAP;
+		strncpy(mtpt, "<swap>", mtlen);
+		return 0;
+	}
 #ifdef HAVE_MNTENT_H
 	return check_mntent(device, mount_flags, mtpt, mtlen);
 #else 
@@ -198,13 +293,16 @@ errcode_t ext2fs_check_if_mounted(const char *file, int *mount_flags)
 int main(int argc, char **argv)
 {
 	int	retval, mount_flags;
+	char	mntpt[80];
 	
 	if (argc < 2) {
 		fprintf(stderr, "Usage: %s device\n", argv[0]);
 		exit(1);
 	}
 
-	retval = ext2fs_check_if_mounted(argv[1], &mount_flags);
+	mntpt[0] = 0;
+	retval = ext2fs_check_mount_point(argv[1], &mount_flags,
+					  mntpt, sizeof(mntpt));
 	if (retval) {
 		com_err(argv[0], retval,
 			"while calling ext2fs_check_if_mounted");
@@ -213,12 +311,17 @@ int main(int argc, char **argv)
 	printf("Device %s reports flags %02x\n", argv[1], mount_flags);
 	if (mount_flags & EXT2_MF_MOUNTED)
 		printf("\t%s is mounted.\n", argv[1]);
-	
+
+	if (mount_flags & EXT2_MF_SWAP)
+		printf("\t%s is a swap device.\n", argv[1]);
+
 	if (mount_flags & EXT2_MF_READONLY)
 		printf("\t%s is read-only.\n", argv[1]);
 	
 	if (mount_flags & EXT2_MF_ISROOT)
 		printf("\t%s is the root filesystem.\n", argv[1]);
+	if (mntpt[0])
+		printf("\t%s is mounted on %s.\n", argv[1], mntpt);
 	
 	exit(0);
 }
