@@ -15,21 +15,16 @@
 #include <unistd.h>
 #endif
 
-#if EXT2_FLAT_INCLUDES
 #include "ext2_fs.h"
-#else
-#include <linux/ext2_fs.h>
-#endif
-
 #include "ext2fs.h"
 
 struct link_struct  {
 	const char	*name;
 	int		namelen;
-	ino_t		inode;
+	ext2_ino_t	inode;
 	int		flags;
 	int		done;
-	struct ext2fs_sb *sb;
+	struct ext2_super_block *sb;
 };	
 
 static int link_proc(struct ext2_dir_entry *dirent,
@@ -97,10 +92,10 @@ static int link_proc(struct ext2_dir_entry *dirent,
  * entry filetype.
  */
 #ifdef __TURBOC__
-#pragma argsused
+ #pragma argsused
 #endif
-errcode_t ext2fs_link(ext2_filsys fs, ino_t dir, const char *name, ino_t ino,
-		      int flags)
+errcode_t ext2fs_link(ext2_filsys fs, ext2_ino_t dir, const char *name, 
+		      ext2_ino_t ino, int flags)
 {
 	errcode_t	retval;
 	struct link_struct ls;
@@ -115,7 +110,7 @@ errcode_t ext2fs_link(ext2_filsys fs, ino_t dir, const char *name, ino_t ino,
 	ls.inode = ino;
 	ls.flags = flags;
 	ls.done = 0;
-	ls.sb = (struct ext2fs_sb *) fs->super;
+	ls.sb = fs->super;
 
 	retval = ext2fs_dir_iterate(fs, dir, DIRENT_FLAG_INCLUDE_EMPTY,
 				    0, link_proc, &ls);
