@@ -23,12 +23,7 @@
 #include <sys/types.h>
 #endif
 
-#if EXT2_FLAT_INCLUDES
 #include "ext2_fs.h"
-#else
-#include <linux/ext2_fs.h>
-#endif
-
 #include "ext2fs.h"
 
 /*
@@ -44,6 +39,7 @@ struct test_private_data {
 	void (*read_blk)(unsigned long block, int count, errcode_t err);
 	void (*write_blk)(unsigned long block, int count, errcode_t err);
 	void (*set_blksize)(int blksize, errcode_t err);
+	void (*write_byte)(unsigned long block, int count, errcode_t err);
 };
 
 static errcode_t test_open(const char *name, int flags, io_channel *channel);
@@ -54,6 +50,8 @@ static errcode_t test_read_blk(io_channel channel, unsigned long block,
 static errcode_t test_write_blk(io_channel channel, unsigned long block,
 				int count, const void *data);
 static errcode_t test_flush(io_channel channel);
+static errcode_t test_write_byte(io_channel channel, unsigned long offset,
+				 int count, const void *buf);
 
 static struct struct_io_manager struct_test_manager = {
 	EXT2_ET_MAGIC_IO_MANAGER,
@@ -63,7 +61,9 @@ static struct struct_io_manager struct_test_manager = {
 	test_set_blksize,
 	test_read_blk,
 	test_write_blk,
-	test_flush
+	test_flush,
+	test_write_byte
+	
 };
 
 io_manager test_io_manager = &struct_test_manager;
@@ -79,6 +79,8 @@ void (*test_io_cb_write_blk)
 	(unsigned long block, int count, errcode_t err) = 0;
 void (*test_io_cb_set_blksize)
 	(int blksize, errcode_t err) = 0;
+void (*test_io_cb_write_byte)
+	(unsigned long block, int count, errcode_t err) = 0;
 
 static errcode_t test_open(const char *name, int flags, io_channel *channel)
 {
@@ -124,6 +126,7 @@ static errcode_t test_open(const char *name, int flags, io_channel *channel)
 	data->read_blk = 	test_io_cb_read_blk;
 	data->write_blk = 	test_io_cb_write_blk;
 	data->set_blksize = 	test_io_cb_set_blksize;
+	data->write_byte = 	test_io_cb_write_byte;
 	
 	*channel = io;
 	return 0;
@@ -151,8 +154,7 @@ static errcode_t test_close(io_channel channel)
 	if (data->real)
 		retval = io_channel_close(data->real);
 	
-	if (channel->private_data)
-		ext2fs_free_mem((void **) &channel->private_data);
+	ext2fs_free_mem((void **) &channel->private_data);
 	if (channel->name)
 		ext2fs_free_mem((void **) &channel->name);
 	ext2fs_free_mem((void **) &channel);
@@ -219,6 +221,26 @@ static errcode_t test_write_blk(io_channel channel, unsigned long block,
 	return retval;
 }
 
+static errcode_t test_write_byte(io_channel channel, unsigned long offset,
+			       int count, const void *buf)
+{
+	struct test_private_data *data;
+	errcode_t	retval = 0;
+
+	EXT2_CHECK_MAGIC(channel, EXT2_ET_MAGIC_IO_CHANNEL);
+	data = (struct test_private_data *) channel->private_data;
+	EXT2_CHECK_MAGIC(data, EXT2_ET_MAGIC_TEST_IO_CHANNEL);
+
+	if (data->real && data->real->manager->write_byte)
+		retval = io_channel_write_byte(data->real, offset, count, buf);
+	if (data->write_byte)
+		data->write_byte(offset, count, retval);
+	else
+		printf("Test_io: write_byte(%lu, %d) returned %s\n",
+		       offset, count, retval ? error_message(retval) : "OK");
+	return retval;
+}
+
 /*
  * Flush data buffers to disk.
  */
