@@ -1,7 +1,7 @@
 /*
  *------------------------------------------------------------------
  *
- * $Source: /usr/src/e2fsprogs/e2fsprogs/lib/ss/SCCS/s.test_ss.c $
+ * $Source: /usr/src/e2fsprogs/BK/e2fsprogs/lib/ss/SCCS/s.test_ss.c $
  * $Revision: 1.13 $
  * $Date: 97/04/29 21:26:37-00:00 $
  * $State:  <unknown> $
@@ -33,7 +33,7 @@
 
 #ifndef lint
 static char const rcsid_test_c[] =
-    "$Header: /usr/src/e2fsprogs/e2fsprogs/lib/ss/SCCS/s.test_ss.c 1.13 97/04/29 21:26:37-00:00 tytso@mit.edu $";
+    "$Header: lib/ss/SCCS/s.test_ss.c 1.13 97/04/29 21:26:37-00:00 tytso@mit.edu $";
 #endif /* lint */
 
 #include <stdio.h>
