@@ -1,14 +1,20 @@
 /*
  * gen_uuid.c --- generate a DCE-compatible uuid
  *
- * Copyright (C) 1996, 1997 Theodore Ts'o.
+ * Copyright (C) 1996, 1997, 1998, 1999 Theodore Ts'o.
  *
  * %Begin-Header%
- * This file may be redistributed under the terms of the GNU Public
- * License.
+ * This file may be redistributed under the terms of the GNU 
+ * Library General Public License.
  * %End-Header%
  */
 
+/*
+ * Force inclusion of SVID stuff since we need it if we're compiling in
+ * gcc-wall wall mode
+ */
+#define _SVID_SOURCE
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -41,40 +47,58 @@
 #define rand() 		random()
 #endif
 
+static int get_random_fd(void)
+{
+	struct timeval	tv;
+	static int	fd = -2;
+	int		i;
+
+	if (fd == -2) {
+		gettimeofday(&tv, 0);
+		fd = open("/dev/urandom", O_RDONLY);
+		if (fd == -1)
+			fd = open("/dev/random", O_RDONLY | O_NONBLOCK);
+		srand((getpid() << 16) ^ getuid() ^ tv.tv_sec ^ tv.tv_usec);
+	}
+	/* Crank the random number generator a few times */
+	gettimeofday(&tv, 0);
+	for (i = (tv.tv_sec ^ tv.tv_usec) & 0x1F; i > 0; i--)
+		rand();
+	return fd;
+}
+
+
 /*
  * Generate a series of random bytes.  Use /dev/urandom if possible,
  * and if not, use srandom/random.
  */
 static void get_random_bytes(void *buf, int nbytes)
 {
-	static int fd = -2;
-	int i;
+	int i, fd = get_random_fd();
+	int lose_counter = 0;
 	char *cp = (char *) buf;
 
-	if (fd == -2) {
-		fd = open("/dev/urandom", O_RDONLY);
-		srand((getpid() << 16) ^ getuid() ^ time(0));
-	}
 	if (fd >= 0) {
 		while (nbytes > 0) {
 			i = read(fd, cp, nbytes);
-			if (i < 0) {
-				if ((errno == EINTR) || (errno == EAGAIN))
-					continue;
-				break;
+			if ((i < 0) &&
+			    ((errno == EINTR) || (errno == EAGAIN)))
+				continue;
+			if (i <= 0) {
+				if (lose_counter++ == 8)
+					break;
+				continue;
 			}
 			nbytes -= i;
 			cp += i;
+			lose_counter = 0;
 		}
 	}
-	if (nbytes == 0)
-		return;
 
 	/* XXX put something better here if no /dev/random! */
-	for (i=0; i < nbytes; i++)
+	for (i = 0; i < nbytes; i++)
 		*cp++ = rand() & 0xFF;
 	return;
-	
 }
 
 /*
@@ -176,14 +200,17 @@ try_again:
 	     (tv.tv_usec < last.tv_usec))) {
 		clock_seq = (clock_seq+1) & 0x1FFF;
 		adjustment = 0;
+		last = tv;
 	} else if ((tv.tv_sec == last.tv_sec) &&
 	    (tv.tv_usec == last.tv_usec)) {
 		if (adjustment >= MAX_ADJUSTMENT)
 			goto try_again;
 		adjustment++;
-	} else
+	} else {
 		adjustment = 0;
-	
+		last = tv;
+	}
+		
 	clock_reg = tv.tv_usec*10 + adjustment;
 	clock_reg += ((unsigned long long) tv.tv_sec)*10000000;
 	clock_reg += (((unsigned long long) 0x01B21DD2) << 32) + 0x13814000;
@@ -242,17 +269,8 @@ void uuid_generate_random(uuid_t out)
  */
 void uuid_generate(uuid_t out)
 {
-	static int	has_random = -1;
-
-	if (has_random < 0) {
-		if (access("/dev/urandom", R_OK) == 0)
-			has_random = 1;
-		else
-			has_random = 0;
-	}
-	if (has_random)
+	if (get_random_fd() >= 0)
 		uuid_generate_random(out);
 	else
 		uuid_generate_time(out);
 }
-
