@@ -96,6 +96,7 @@ static const struct flags_char flags_array[] = {
 	{ EXT3_JOURNAL_DATA_FL, 'j' },
 	{ EXT2_SECRM_FL, 's' },
 	{ EXT2_UNRM_FL, 'u' },
+	{ EXT2_NOTAIL_FL, 't' },
 	{ 0, 0 }
 };
 
@@ -257,6 +258,7 @@ int main (int argc, char ** argv)
 
 #ifdef ENABLE_NLS
 	setlocale(LC_MESSAGES, "");
+	setlocale(LC_CTYPE, "");
 	bindtextdomain(NLS_CAT_NAME, LOCALEDIR);
 	textdomain(NLS_CAT_NAME);
 #endif
@@ -284,9 +286,8 @@ int main (int argc, char ** argv)
 		exit (1);
 	}
 	if (verbose)
-		fprintf (stderr, _("chattr %s, %s for EXT2 FS %s, %s\n"),
-			 E2FSPROGS_VERSION, E2FSPROGS_DATE,
-			 EXT2FS_VERSION, EXT2FS_DATE);
+		fprintf (stderr, "chattr %s (%s)\n",
+			 E2FSPROGS_VERSION, E2FSPROGS_DATE);
 	for (j = i; j < argc; j++)
 		change_attributes (argv[j]);
 	exit(0);
