@@ -35,7 +35,7 @@
 
 #include "resize2fs.h"
 
-#ifdef linux			/* Kludge for debugging */
+#ifdef __linux__			/* Kludge for debugging */
 #define RESIZE2FS_DEBUG
 #endif
 
@@ -881,7 +881,7 @@ static int process_block(ext2_filsys fs, blk_t	*block_nr,
 			pb->changed = 1;
 #ifdef RESIZE2FS_DEBUG
 			if (pb->rfs->flags & RESIZE_DEBUG_BMOVE)
-				printf(_("ino=%ld, blockcnt=%ld, %u->%u\n"), 
+				printf(_("ino=%u, blockcnt=%lld, %u->%u\n"), 
 				       pb->ino, blockcnt, block, new_block);
 #endif
 			block = new_block;
@@ -1037,7 +1037,7 @@ static errcode_t inode_scan_and_fix(ext2_resize_t rfs)
 		
 #ifdef RESIZE2FS_DEBUG
 		if (rfs->flags & RESIZE_DEBUG_INODEMAP)
-			printf(_("Inode moved %ld->%ld\n"), ino, new_inode);
+			printf(_("Inode moved %u->%u\n"), ino, new_inode);
 #endif
 		if (!rfs->imap) {
 			retval = ext2fs_create_extent_table(&rfs->imap, 0);
@@ -1100,7 +1100,7 @@ static int check_and_change_inodes(ext2_ino_t dir, int entry,
 		return 0;
 #ifdef RESIZE2FS_DEBUG
 	if (is->rfs->flags & RESIZE_DEBUG_INODEMAP)
-		printf(_("Inode translate (dir=%ld, name=%.*s, %u->%ld)\n"),
+		printf(_("Inode translate (dir=%u, name=%.*s, %u->%u)\n"),
 		       dir, dirent->name_len, dirent->name,
 		       dirent->inode, new_inode);
 #endif
